/* original parser id follows */
/* yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93" */
/* (use YYMAJOR/YYMINOR for ifdefs dependent on parser version) */

#define YYBYACC 1
#define YYMAJOR 2
#define YYMINOR 0
#define YYPATCH 20230219

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)
#define YYENOMEM       (-2)
#define YYEOF          0
#undef YYBTYACC
#define YYBTYACC 0
#define YYDEBUGSTR YYPREFIX "debug"
#define YYPREFIX "yy"

#define YYPURE 0

#line 33 "grammar.y"

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdint.h>
#include <netinet/in.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>

#include "util.h"
#include "output_util.h"
#include "rbtree.h"
#include "filter.h"
#include "nfdump.h"
#include "nffile.h"
#include "nftree.h"
#include "ipconv.h"
#include "sgregex/sgregex.h"

#define AnyMask 0xffffffffffffffffLL

/*
 * function prototypes
 */
static void  yyerror(char *msg);

static uint32_t ChainHosts(uint64_t *offsets, uint64_t *hostlist, int num_records, int type);

static uint64_t VerifyMac(char *s);

static int InitSymbols(void);

static uint32_t Get_fwd_status_id(char *status);

static char *stripWord(char *word);

static int IsMD5(char *string);

enum { DIR_UNSPEC = 1, 
	   SOURCE, DESTINATION, SOURCE_AND_DESTINATION, SOURCE_OR_DESTINATION, 
	   DIR_IN, DIR_OUT, 
	   IN_SRC, IN_DST, OUT_SRC, OUT_DST, 
	   ADJ_PREV, ADJ_NEXT };

enum { IS_START = 0, IS_END };

/* var defs */
extern int 			lineno;
extern char 		*yytext;
extern uint64_t		*IPstack;
extern uint32_t	StartNode;
extern uint8_t	geoFilter;
extern uint8_t	ja3Filter;
extern int (*FilterEngine)(uint32_t *);
extern char	*FilterFilename;

static uint32_t num_ip;

static struct fwd_status_def_s {
	uint32_t	id;
	char		*name;
} fwd_status_def_list[] = {
	{ 0,	"Ukwn"}, 	/* Unknown*/
	{ 1,	"Forw"}, 	/* Normal forwarding*/
	{ 2,	"Frag"}, 	/* Fragmented*/
	{ 16,	"Drop"}, 	/* Drop*/
	{ 17,	"DaclD"},	/* Drop ACL deny*/
	{ 18,	"Daclp"},	/* Drop ACL drop*/
	{ 19,	"Noroute"},	/* Unroutable*/
	{ 20,	"Dadj"}, 	/* Drop Adjacency*/
	{ 21,	"Dfrag"}, 	/* Drop Fragmentation & DF set*/
	{ 22,	"Dbadh"}, 	/* Drop Bad header checksum*/
	{ 23,	"Dbadtlen"}, /* Drop Bad total Length*/
	{ 24,	"Dbadhlen"}, /* Drop Bad Header Length*/
	{ 25,	"DbadTTL"}, /* Drop bad TTL*/
	{ 26,	"Dpolicy"}, /* Drop Policer*/
	{ 27,	"Dwred"}, 	/* Drop WRED*/
	{ 28,	"Drpf"}, 	/* Drop RPF*/
	{ 29,	"Dforus"}, 	/* Drop For us*/
	{ 30,	"DbadOf"}, 	/* Drop Bad output interface*/
	{ 31,	"Dhw"}, 	/* Drop Hardware*/
	{ 128,	"Term"}, 	/* Terminate*/
	{ 129,	"Tadj"}, 	/* Terminate Punt Adjacency*/
	{ 130,	"TincAdj"}, /* Terminate Incomplete Adjacency*/
	{ 131,	"Tforus"}, 	/* Terminate For us*/
	{ 0,	NULL}		/* Last entry*/
};

static char **fwd_status = NULL;

char yyerror_buff[256];

#define MPLSMAX 0x00ffffff
#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#line 132 "grammar.y"
typedef union YYSTYPE {
	uint64_t		value;
	char			*s;
	FilterParam_t	param;
	void			*list;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */
#line 136 "grammar.c"

/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex(void)
# define YYLEX yylex()
#endif

#if !(defined(yylex) || defined(YYSTATE))
int YYLEX_DECL();
#endif

/* Parameters sent to yyerror. */
#ifndef YYERROR_DECL
#define YYERROR_DECL() yyerror(const char *s)
#endif
#ifndef YYERROR_CALL
#define YYERROR_CALL(msg) yyerror(msg)
#endif

extern int YYPARSE_DECL();

#define ANY 257
#define IP 258
#define TUNIP 259
#define IF 260
#define MAC 261
#define MPLS 262
#define TOS 263
#define DIR 264
#define FLAGS 265
#define TUN 266
#define PROTO 267
#define MASK 268
#define NET 269
#define PORT 270
#define FWDSTAT 271
#define IN 272
#define OUT 273
#define SRC 274
#define DST 275
#define EQ 276
#define LT 277
#define GT 278
#define LE 279
#define GE 280
#define PREV 281
#define NEXT 282
#define IDENT 283
#define ENGINE_TYPE 284
#define ENGINE_ID 285
#define EXPORTER 286
#define AS 287
#define GEO 288
#define PACKETS 289
#define BYTES 290
#define FLOWS 291
#define LABEL 292
#define NFVERSION 293
#define COUNT 294
#define PPS 295
#define BPS 296
#define BPP 297
#define DURATION 298
#define NOT 299
#define IPV4 300
#define IPV6 301
#define BGPNEXTHOP 302
#define ROUTER 303
#define VLAN 304
#define CLIENT 305
#define SERVER 306
#define APP 307
#define LATENCY 308
#define SYSID 309
#define ASA 310
#define DENIED 311
#define XEVENT 312
#define XNET 313
#define XPORT 314
#define INGRESS 315
#define EGRESS 316
#define ACL 317
#define ACE 318
#define XACE 319
#define NAT 320
#define ADD 321
#define EVENT 322
#define VRF 323
#define NPORT 324
#define NIP 325
#define PBLOCK 326
#define START 327
#define END 328
#define STEP 329
#define SIZE 330
#define PAYLOAD 331
#define CONTENT 332
#define REGEX 333
#define JA3 334
#define OBSERVATION 335
#define DOMAIN 336
#define POINT 337
#define ID 338
#define PF 339
#define PFACTION 340
#define PFREASON 341
#define RULE 342
#define INTERFACE 343
#define STRING 344
#define WORD 345
#define REASON 346
#define NUMBER 347
#define PORTNUM 348
#define ICMP_TYPE 349
#define ICMP_CODE 350
#define OR 351
#define AND 352
#define NEGATE 353
#define YYERRCODE 256
typedef int YYINT;
static const YYINT yylhs[] = {                           -1,
    0,    0,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    7,    7,    7,    7,    7,    8,    8,    8,    4,
    4,    4,    4,    4,    4,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    6,    6,    5,    5,    5,    1,    1,    1,    1,    1,
    1,    1,
};
static const YYINT yylen[] = {                            2,
    0,    1,    1,    2,    1,    1,    2,    2,    3,    3,
    4,    4,    3,    3,    3,    3,    3,    3,    4,    3,
    3,    2,    2,    3,    3,    6,    3,    6,    3,    3,
    4,    4,    4,    2,    4,    4,    4,    3,    3,    3,
    3,    3,    3,    3,    4,    6,    2,    2,    3,    3,
    3,    2,    3,    4,    4,    4,    4,    5,    4,    4,
    3,    4,    4,    4,    4,    4,    4,    3,    3,    3,
    3,    4,    3,    6,    4,    3,    6,    3,    4,    5,
    3,    3,    3,    4,    3,    2,    2,    2,    2,    4,
    4,    1,    3,    2,    3,    4,    1,    2,    3,    0,
    1,    1,    1,    1,    1,    0,    1,    1,    3,    3,
    3,    3,    1,    1,    2,    2,    2,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    3,    3,    2,    3,
    5,    5,
};
static const YYINT yydefred[] = {                         0,
    3,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  119,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    5,    6,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  126,
    0,    0,    0,  121,  122,   88,   89,  101,  102,  103,
  104,  105,   22,   23,    0,    0,    8,    7,   87,   86,
  115,  116,  117,  118,    0,    0,    0,    0,    0,    4,
    0,    0,    0,    0,   52,    0,    0,    0,    0,    0,
    0,  129,    0,    0,    0,    0,    0,   34,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   47,   48,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  123,  124,  125,    0,   85,    0,   21,   10,    9,  109,
  111,  110,  112,    0,   27,   49,   50,   51,   13,   20,
   18,   14,   15,   16,   17,   29,   30,    0,    0,    0,
    0,    0,   53,    0,    0,    0,   61,    0,    0,    0,
    0,    0,   69,   70,    0,   73,    0,    0,   43,   44,
   39,   38,   40,   42,   41,    0,    0,    0,  128,    0,
   24,   25,   81,   83,    0,   78,    0,    0,    0,    0,
    0,   76,    0,    0,   82,    0,    0,    0,    0,   68,
    0,   84,    0,   31,   32,   33,   57,   56,   55,   54,
   36,   37,   62,   63,   64,   65,   66,   72,   90,   91,
    0,    0,    0,   19,   79,    0,   45,    0,   35,    0,
   75,   11,   12,    0,   59,   67,    0,   60,    0,    0,
  131,  132,    0,   80,   97,    0,    0,   58,    0,    0,
    0,   28,    0,   26,   98,   46,    0,   77,   74,   93,
    0,   95,   99,   96,
};
#if defined(YYDESTRUCT_CALL) || defined(YYSTYPE_TOSTRING)
static const YYINT yystos[] = {                           0,
  257,  262,  264,  265,  266,  267,  271,  272,  273,  274,
  275,  281,  282,  283,  284,  285,  286,  291,  292,  293,
  294,  295,  296,  297,  298,  299,  300,  301,  302,  303,
  305,  306,  307,  309,  310,  315,  316,  320,  326,  331,
  335,  339,  349,  350,   40,   37,  355,  356,  357,  358,
  361,  257,  344,  315,  316,  347,  361,  276,  277,  278,
  279,  280,  287,  344,  359,  267,  344,  347,  344,  347,
  274,  275,  274,  275,  351,  352,  351,  352,  258,  344,
  359,  359,  359,  359,  344,  359,  359,  359,  359,  359,
  359,  356,  258,  258,  308,  308,  308,  347,  312,  322,
  323,  323,  322,  327,  328,  329,  330,  332,  333,  334,
  336,  337,  264,  340,  341,  342,  343,  347,  347,  356,
  344,  351,  352,  258,  259,  260,  261,  263,  268,  269,
  270,  287,  288,  289,  290,  304,  313,  314,  324,  325,
  317,  318,  319,  360,  347,  359,  347,  344,  347,  275,
  275,  274,  274,  272,  344,  347,  347,  347,  347,  347,
  347,  347,  347,  347,  347,  344,  344,  359,  359,  359,
  359,  311,  346,  359,  359,  359,  346,  359,  359,  359,
  359,  359,  344,  345,  345,  344,  338,  338,  272,  273,
  320,  344,  344,  347,  344,   41,   40,  356,  356,  272,
  344,  344,  347,  344,  359,  347,  344,  272,  359,  272,
  359,  344,  359,  359,  347,  344,  359,  359,  272,  344,
  359,  347,   91,  347,  347,  347,  347,  344,  361,  347,
  347,  347,  347,  347,  347,  347,  347,  344,  347,  347,
   37,  356,   91,  347,  344,   47,  326,   91,  347,   91,
  347,  347,  347,   47,  347,  347,   91,  347,  344,  362,
  344,   41,  362,  347,  347,  363,  363,  347,  362,   47,
  344,   93,   44,   93,  347,   93,   44,   93,   93,  347,
   47,  344,  347,  347,
};
#endif /* YYDESTRUCT_CALL || YYSTYPE_TOSTRING */
static const YYINT yydgoto[] = {                         47,
   48,   49,   50,   65,  144,   51,  260,  266,
};
static const YYINT yysindex[] = {                        76,
    0, -242, -295, -229, -245, -278, -262, -187, -176, -188,
 -174,    0, -234, -318, -107, -107, -107, -107, -303, -107,
 -107, -107, -107, -107, -107,   76,    0,    0, -215, -204,
 -227, -217, -212, -287, -238, -256, -222, -205, -218, -208,
 -233, -248, -207, -202,   76, -209,    0, -167, -122,    0,
 -185, -194, -107,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -168, -261,    0,    0,    0,    0,
    0,    0,    0,    0, -126,  -87,  -79,  -76, -255,    0,
 -142, -141, -140, -139,    0, -138, -137, -132, -130, -129,
 -128,    0, -124, -120, -107, -107, -107,    0, -107, -246,
 -107, -107, -241, -107, -107, -107, -107, -148, -119, -117,
 -134, -113,  -72, -280, -116, -118, -114,    0,    0,  -41,
  191,   76,   76, -254, -112, -111, -110, -107, -109, -105,
 -158, -149,  -99, -107, -107, -101,  -97, -107, -107, -247,
    0,    0,    0, -107,    0,  -98,    0,    0,    0,    0,
    0,    0,    0,  142,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  -96,  -95,  -94,
  -93, -288,    0,  -92,  -91,  -89,    0,  -88,  -83,  -78,
  -74,  -73,    0,    0,  -69,    0,  -71,  -68,    0,    0,
    0,    0,    0,    0,    0,  198,   76, -115,    0,  157,
    0,    0,    0,    0,  -67,    0,  -24,  -82,  -66,  159,
  -65,    0,  -64,  -63,    0,  221,  -62,  -61,  196,    0,
  -59,    0,  -51,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  -50,  -34,  -51,    0,    0,  -52,    0,  -49,    0,  -49,
    0,    0,    0,  -48,    0,    0,  -51,    0,  249,  -36,
    0,    0,  -32,    0,    0,  -40,  -38,    0,  -31,  -46,
  250,    0,  -44,    0,    0,    0,  -45,    0,    0,    0,
  -43,    0,    0,    0,
};
static const YYINT yyrindex[] = {                         2,
    0,    0,    0,  -42,    0,    0,    0,  -47,  140,  173,
  212,    0,  284,    0,  -42,  -42,  -42,  -42,    0,  -42,
  -42,  -42,  -42,  -42,  -42,  245,    0,    0,    0,    0,
    0,    0,    0,    0,    0, -175, -143,    0,    0,    0,
    0,    0,    0,    0,  245,    0,    0,  303,    0,    0,
    0,    0,  -42,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  -42,  -42,  -42,    0,  -42,  -42,
  -42,  -42,  -42,  -42,  -42,  -42,  -42,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  245,  245,    0,    0,    0,    0,  -42,    0,    0,
  -42,  -42,    0,  -42,  -42,    0,    0,  -42,  -42,    0,
    0,    0,    0,  -42,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    1,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    3,  245,    5,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  -30,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  -25,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,
};
#if YYBTYACC
static const YYINT yycindex[] = {                         0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,
};
#endif
static const YYINT yygindex[] = {                         0,
  -16,    0,    0,   55,    0,    8, -198,   71,
};
#define YYTABLESIZE 609
static const YYINT yytable[] = {                        196,
   71,    1,  130,  277,  127,  277,  262,  273,  248,   92,
   57,  273,  273,   92,   52,  113,  154,  200,   94,   54,
   55,   66,  246,   79,  219,   80,   54,   55,  120,   58,
   59,   60,   61,   62,   58,   59,   60,   61,   62,  191,
   85,   71,   93,  130,  263,  127,   58,   59,   60,   61,
   62,   56,  276,   94,  278,  228,  272,   63,  269,   98,
  274,  279,   92,  192,  172,   67,  101,   94,   68,   81,
   82,   83,   84,   99,   86,   87,   88,   89,   90,   91,
   95,   69,  148,  100,   70,  149,   71,   72,  155,  201,
   96,  114,  115,  116,  117,   97,  220,   73,   74,  173,
  102,   53,  111,  112,  177,  198,  199,  146,  104,  105,
  106,  107,   46,  208,   64,   45,  103,   58,   59,   60,
   61,   62,  210,  108,  109,  110,   58,   59,   60,   61,
   62,  141,  142,  143,  121,  124,  125,  126,  127,  118,
  128,  121,  121,  121,  119,  129,  130,  131,  150,  168,
  169,  170,  145,  171,  174,  175,  176,  178,  179,  180,
  181,  182,   75,   76,  132,  133,  134,  135,   58,   59,
   60,   61,   62,  122,  122,  122,   77,   78,  147,  229,
  242,  136,  205,  122,  123,  209,  211,  151,  213,  214,
  137,  138,  217,  218,  152,  183,  184,  153,  221,  189,
  190,  139,  140,  187,  156,  157,  158,  159,  160,  161,
  113,  113,  113,  113,  162,  113,  163,  164,  165,  166,
  113,  113,  113,  167,  188,  185,  186,  193,  194,  195,
  197,  202,  223,  204,  241,  203,  123,  206,  207,  113,
  113,  113,  113,  247,  212,  215,  216,  243,  222,  250,
  224,  225,  226,  227,  230,  231,  113,  232,  233,  106,
  106,  106,  106,  234,  106,  113,  113,  254,  235,  106,
  106,  106,  236,  237,  238,  239,  113,  113,  240,  244,
  249,  251,  252,  253,  255,  256,  257,  258,  106,  106,
  106,  106,  259,  261,  264,  270,  281,  265,  268,  282,
  280,  283,    2,  284,  100,  106,  275,  271,  275,  122,
  123,  271,  271,   92,  106,  106,  122,  123,   94,  245,
  267,    0,    0,    0,    0,  106,  106,    0,    0,    0,
    0,    0,    1,    0,    0,    0,    0,    2,    0,    3,
    4,    5,    6,    0,    0,    0,    7,    8,    9,   10,
   11,   71,   71,  130,  130,  127,   12,   13,   14,   15,
   16,   17,    0,    0,    0,    0,   18,   19,   20,   21,
   22,   23,   24,   25,   26,   27,   28,   29,   30,    0,
   31,   32,   33,    0,   34,   35,    0,    0,    0,    0,
   36,   37,    0,    0,    0,   38,    0,  114,  114,  114,
  114,   39,  114,    0,    0,    0,   40,  114,  114,  114,
   41,    0,    0,    0,   42,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   43,   44,  114,  114,  114,  114,
  107,  107,  107,  107,    0,  107,    0,    0,    0,    0,
  107,  107,  107,  114,    0,    0,    0,    0,    0,    0,
    0,    0,  114,  114,    0,    0,    0,    0,    0,  107,
  107,  107,  107,  114,  114,    0,    0,    0,    0,  108,
  108,  108,  108,    0,  108,    0,  107,    0,    0,  108,
  108,  108,    0,    0,    0,  107,  107,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  107,  107,  108,  108,
  108,  108,  106,  106,  106,  106,    0,  106,    0,    0,
    0,    0,  106,  106,  106,  108,    0,    0,    0,    0,
    0,    0,    0,    0,  108,  108,    0,    0,    0,    0,
    0,  106,  106,  106,  106,  108,  108,    0,    0,    0,
    0,    0,  120,  120,  120,    0,  120,    0,  106,    0,
    0,  120,  120,  120,    0,    0,    0,  106,  106,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  106,  106,
  120,  120,  120,  120,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  120,    0,    0,
    0,    0,    0,    0,    0,    0,  120,  120,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  120,  120,
};
static const YYINT yycheck[] = {                         41,
    0,    0,    0,   44,    0,   44,   41,   44,   91,   26,
    3,   44,   44,   44,  257,  264,  272,  272,   44,  315,
  316,  267,   47,  258,  272,  344,  315,  316,   45,  276,
  277,  278,  279,  280,  276,  277,  278,  279,  280,  320,
  344,   41,  258,   41,  243,   41,  276,  277,  278,  279,
  280,  347,   93,  258,   93,  344,   93,  287,  257,  347,
   93,   93,   93,  344,  311,  344,  323,   93,  347,   15,
   16,   17,   18,  312,   20,   21,   22,   23,   24,   25,
  308,  344,  344,  322,  347,  347,  274,  275,  344,  344,
  308,  340,  341,  342,  343,  308,  344,  274,  275,  346,
  323,  344,  336,  337,  346,  122,  123,   53,  327,  328,
  329,  330,   37,  272,  344,   40,  322,  276,  277,  278,
  279,  280,  272,  332,  333,  334,  276,  277,  278,  279,
  280,  317,  318,  319,  344,  258,  259,  260,  261,  347,
  263,  317,  318,  319,  347,  268,  269,  270,  275,   95,
   96,   97,  347,   99,  100,  101,  102,  103,  104,  105,
  106,  107,  351,  352,  287,  288,  289,  290,  276,  277,
  278,  279,  280,  317,  318,  319,  351,  352,  347,  172,
  197,  304,  128,  351,  352,  131,  132,  275,  134,  135,
  313,  314,  138,  139,  274,  344,  345,  274,  144,  272,
  273,  324,  325,  338,  347,  347,  347,  347,  347,  347,
  258,  259,  260,  261,  347,  263,  347,  347,  347,  344,
  268,  269,  270,  344,  338,  345,  344,  344,  347,  344,
   40,  344,   91,  344,   37,  347,  352,  347,  344,  287,
  288,  289,  290,  326,  344,  347,  344,   91,  347,   91,
  347,  347,  347,  347,  347,  347,  304,  347,  347,  258,
  259,  260,  261,  347,  263,  313,  314,   47,  347,  268,
  269,  270,  347,  347,  344,  347,  324,  325,  347,  347,
  347,  347,  347,  347,  347,  347,   91,  347,  287,  288,
  289,  290,  344,  344,  347,   47,   47,  347,  347,  344,
  347,  347,    0,  347,  347,  304,  347,  344,  347,  351,
  352,  344,  344,  344,  313,  314,  351,  352,  344,  344,
  250,   -1,   -1,   -1,   -1,  324,  325,   -1,   -1,   -1,
   -1,   -1,  257,   -1,   -1,   -1,   -1,  262,   -1,  264,
  265,  266,  267,   -1,   -1,   -1,  271,  272,  273,  274,
  275,  351,  352,  351,  352,  351,  281,  282,  283,  284,
  285,  286,   -1,   -1,   -1,   -1,  291,  292,  293,  294,
  295,  296,  297,  298,  299,  300,  301,  302,  303,   -1,
  305,  306,  307,   -1,  309,  310,   -1,   -1,   -1,   -1,
  315,  316,   -1,   -1,   -1,  320,   -1,  258,  259,  260,
  261,  326,  263,   -1,   -1,   -1,  331,  268,  269,  270,
  335,   -1,   -1,   -1,  339,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  349,  350,  287,  288,  289,  290,
  258,  259,  260,  261,   -1,  263,   -1,   -1,   -1,   -1,
  268,  269,  270,  304,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  313,  314,   -1,   -1,   -1,   -1,   -1,  287,
  288,  289,  290,  324,  325,   -1,   -1,   -1,   -1,  258,
  259,  260,  261,   -1,  263,   -1,  304,   -1,   -1,  268,
  269,  270,   -1,   -1,   -1,  313,  314,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  324,  325,  287,  288,
  289,  290,  258,  259,  260,  261,   -1,  263,   -1,   -1,
   -1,   -1,  268,  269,  270,  304,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  313,  314,   -1,   -1,   -1,   -1,
   -1,  287,  288,  289,  290,  324,  325,   -1,   -1,   -1,
   -1,   -1,  259,  260,  261,   -1,  263,   -1,  304,   -1,
   -1,  268,  269,  270,   -1,   -1,   -1,  313,  314,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  324,  325,
  287,  288,  289,  290,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  304,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  313,  314,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  324,  325,
};
#if YYBTYACC
static const YYINT yyctable[] = {                        -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
};
#endif
#define YYFINAL 47
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 353
#define YYUNDFTOKEN 364
#define YYTRANSLATE(a) ((a) > YYMAXTOKEN ? YYUNDFTOKEN : (a))
#if YYDEBUG
static const char *const yyname[] = {

"$end",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"'%'",0,0,"'('","')'","'*'","'+'","','",0,0,"'/'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'['",0,"']'",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"error","ANY","IP","TUNIP","IF","MAC","MPLS","TOS","DIR","FLAGS","TUN","PROTO",
"MASK","NET","PORT","FWDSTAT","IN","OUT","SRC","DST","EQ","LT","GT","LE","GE",
"PREV","NEXT","IDENT","ENGINE_TYPE","ENGINE_ID","EXPORTER","AS","GEO","PACKETS",
"BYTES","FLOWS","LABEL","NFVERSION","COUNT","PPS","BPS","BPP","DURATION","NOT",
"IPV4","IPV6","BGPNEXTHOP","ROUTER","VLAN","CLIENT","SERVER","APP","LATENCY",
"SYSID","ASA","DENIED","XEVENT","XNET","XPORT","INGRESS","EGRESS","ACL","ACE",
"XACE","NAT","ADD","EVENT","VRF","NPORT","NIP","PBLOCK","START","END","STEP",
"SIZE","PAYLOAD","CONTENT","REGEX","JA3","OBSERVATION","DOMAIN","POINT","ID",
"PF","PFACTION","PFREASON","RULE","INTERFACE","STRING","WORD","REASON","NUMBER",
"PORTNUM","ICMP_TYPE","ICMP_CODE","OR","AND","NEGATE","$accept","prog","expr",
"dqual","term","comp","acl","inout","iplist","ullist","illegal-symbol",
};
static const char *const yyrule[] = {
"$accept : prog",
"prog :",
"prog : expr",
"term : ANY",
"term : IDENT STRING",
"term : IPV4",
"term : IPV6",
"term : PROTO NUMBER",
"term : PROTO STRING",
"term : TUN PROTO NUMBER",
"term : TUN PROTO STRING",
"term : dqual PACKETS comp NUMBER",
"term : dqual BYTES comp NUMBER",
"term : FLOWS comp NUMBER",
"term : PPS comp NUMBER",
"term : BPS comp NUMBER",
"term : BPP comp NUMBER",
"term : DURATION comp NUMBER",
"term : COUNT comp NUMBER",
"term : dqual TOS comp NUMBER",
"term : NFVERSION comp NUMBER",
"term : FLAGS comp NUMBER",
"term : FLAGS AS",
"term : FLAGS STRING",
"term : dqual IP STRING",
"term : dqual TUNIP STRING",
"term : dqual IP IN '[' iplist ']'",
"term : NEXT IP STRING",
"term : NEXT IP IN '[' iplist ']'",
"term : BGPNEXTHOP IP STRING",
"term : ROUTER IP STRING",
"term : CLIENT LATENCY comp NUMBER",
"term : SERVER LATENCY comp NUMBER",
"term : APP LATENCY comp NUMBER",
"term : SYSID NUMBER",
"term : dqual PORT comp NUMBER",
"term : INGRESS VRF comp NUMBER",
"term : EGRESS VRF comp NUMBER",
"term : PF PFACTION STRING",
"term : PF PFACTION NAT",
"term : PF PFREASON STRING",
"term : PF INTERFACE STRING",
"term : PF RULE NUMBER",
"term : PF DIR IN",
"term : PF DIR OUT",
"term : dqual PORT IN PBLOCK",
"term : dqual PORT IN '[' ullist ']'",
"term : ICMP_TYPE NUMBER",
"term : ICMP_CODE NUMBER",
"term : ENGINE_TYPE comp NUMBER",
"term : ENGINE_ID comp NUMBER",
"term : EXPORTER comp NUMBER",
"term : LABEL STRING",
"term : ASA EVENT REASON",
"term : ASA EVENT comp NUMBER",
"term : ASA EVENT DENIED inout",
"term : ASA EVENT DENIED STRING",
"term : ASA XEVENT comp NUMBER",
"term : dqual XNET STRING '/' NUMBER",
"term : dqual XPORT comp NUMBER",
"term : inout acl comp NUMBER",
"term : NAT EVENT REASON",
"term : NAT EVENT comp NUMBER",
"term : PBLOCK START comp NUMBER",
"term : PBLOCK END comp NUMBER",
"term : PBLOCK STEP comp NUMBER",
"term : PBLOCK SIZE comp NUMBER",
"term : dqual NPORT comp NUMBER",
"term : dqual NIP STRING",
"term : PAYLOAD CONTENT STRING",
"term : PAYLOAD CONTENT WORD",
"term : PAYLOAD REGEX WORD",
"term : PAYLOAD REGEX WORD STRING",
"term : PAYLOAD JA3 STRING",
"term : dqual NIP IN '[' iplist ']'",
"term : dqual AS comp NUMBER",
"term : dqual GEO STRING",
"term : dqual AS IN '[' ullist ']'",
"term : dqual MASK NUMBER",
"term : dqual NET STRING STRING",
"term : dqual NET STRING '/' NUMBER",
"term : dqual IF NUMBER",
"term : dqual VLAN NUMBER",
"term : dqual MAC STRING",
"term : MPLS STRING comp NUMBER",
"term : MPLS ANY NUMBER",
"term : FWDSTAT NUMBER",
"term : FWDSTAT STRING",
"term : DIR NUMBER",
"term : DIR inout",
"term : OBSERVATION DOMAIN ID NUMBER",
"term : OBSERVATION POINT ID NUMBER",
"iplist : STRING",
"iplist : STRING '/' NUMBER",
"iplist : iplist STRING",
"iplist : iplist ',' STRING",
"iplist : iplist STRING '/' NUMBER",
"ullist : NUMBER",
"ullist : ullist NUMBER",
"ullist : ullist ',' NUMBER",
"comp :",
"comp : EQ",
"comp : LT",
"comp : GT",
"comp : LE",
"comp : GE",
"dqual :",
"dqual : SRC",
"dqual : DST",
"dqual : SRC OR DST",
"dqual : DST OR SRC",
"dqual : SRC AND DST",
"dqual : DST AND SRC",
"dqual : IN",
"dqual : OUT",
"dqual : IN SRC",
"dqual : IN DST",
"dqual : OUT SRC",
"dqual : OUT DST",
"dqual : PREV",
"dqual : NEXT",
"inout : INGRESS",
"inout : EGRESS",
"acl : ACL",
"acl : ACE",
"acl : XACE",
"expr : term",
"expr : expr OR expr",
"expr : expr AND expr",
"expr : NOT expr",
"expr : '(' expr ')'",
"expr : '(' expr ')' '%' STRING",
"expr : '%' STRING '(' expr ')'",

};
#endif

#if YYDEBUG
int      yydebug;
#endif

int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;
int      yynerrs;

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
YYLTYPE  yyloc; /* position returned by actions */
YYLTYPE  yylloc; /* position from the lexer */
#endif

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
#ifndef YYLLOC_DEFAULT
#define YYLLOC_DEFAULT(loc, rhs, n) \
do \
{ \
    if (n == 0) \
    { \
        (loc).first_line   = YYRHSLOC(rhs, 0).last_line; \
        (loc).first_column = YYRHSLOC(rhs, 0).last_column; \
        (loc).last_line    = YYRHSLOC(rhs, 0).last_line; \
        (loc).last_column  = YYRHSLOC(rhs, 0).last_column; \
    } \
    else \
    { \
        (loc).first_line   = YYRHSLOC(rhs, 1).first_line; \
        (loc).first_column = YYRHSLOC(rhs, 1).first_column; \
        (loc).last_line    = YYRHSLOC(rhs, n).last_line; \
        (loc).last_column  = YYRHSLOC(rhs, n).last_column; \
    } \
} while (0)
#endif /* YYLLOC_DEFAULT */
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
#if YYBTYACC

#ifndef YYLVQUEUEGROWTH
#define YYLVQUEUEGROWTH 32
#endif
#endif /* YYBTYACC */

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#ifndef YYINITSTACKSIZE
#define YYINITSTACKSIZE 200
#endif

typedef struct {
    unsigned stacksize;
    YYINT    *s_base;
    YYINT    *s_mark;
    YYINT    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    YYLTYPE  *p_base;
    YYLTYPE  *p_mark;
#endif
} YYSTACKDATA;
#if YYBTYACC

struct YYParseState_s
{
    struct YYParseState_s *save;    /* Previously saved parser state */
    YYSTACKDATA            yystack; /* saved parser stack */
    int                    state;   /* saved parser state */
    int                    errflag; /* saved error recovery status */
    int                    lexeme;  /* saved index of the conflict lexeme in the lexical queue */
    YYINT                  ctry;    /* saved index in yyctable[] for this conflict */
};
typedef struct YYParseState_s YYParseState;
#endif /* YYBTYACC */
/* variables for the parser stack */
static YYSTACKDATA yystack;
#if YYBTYACC

/* Current parser state */
static YYParseState *yyps = 0;

/* yypath != NULL: do the full parse, starting at *yypath parser state. */
static YYParseState *yypath = 0;

/* Base of the lexical value queue */
static YYSTYPE *yylvals = 0;

/* Current position at lexical value queue */
static YYSTYPE *yylvp = 0;

/* End position of lexical value queue */
static YYSTYPE *yylve = 0;

/* The last allocated position at the lexical value queue */
static YYSTYPE *yylvlim = 0;

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
/* Base of the lexical position queue */
static YYLTYPE *yylpsns = 0;

/* Current position at lexical position queue */
static YYLTYPE *yylpp = 0;

/* End position of lexical position queue */
static YYLTYPE *yylpe = 0;

/* The last allocated position at the lexical position queue */
static YYLTYPE *yylplim = 0;
#endif

/* Current position at lexical token queue */
static YYINT  *yylexp = 0;

static YYINT  *yylexemes = 0;
#endif /* YYBTYACC */
#line 2462 "grammar.y"

static void  yyerror(char *msg) {

	if ( FilterFilename )
		snprintf(yyerror_buff, 255 ,"File '%s' line %d: %s at '%s'", FilterFilename, lineno, msg, yytext);
	else 
		snprintf(yyerror_buff, 255, "Line %d: %s at '%s'", lineno, msg, yytext);

	yyerror_buff[255] = '\0';
	fprintf(stderr, "%s\n", yyerror_buff);

} /* End of yyerror */

static uint32_t ChainHosts(uint64_t *offsets, uint64_t *hostlist, int num_records, int type) {
uint32_t offset_a, offset_b, i, j, block;
	if ( type == SOURCE ) {
		offset_a = offsets[0];
		offset_b = offsets[1];
	} else {
		offset_a = offsets[2];
		offset_b = offsets[3];
	}

	i = 0;
	block = Connect_AND(
				NewBlock(offset_b, MaskIPv6, hostlist[i+1] , CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(offset_a, MaskIPv6, hostlist[i] , CMP_EQ, FUNC_NONE, NULL )
			);
	i += 2;
	for ( j=1; j<num_records; j++ ) {
		uint32_t b = Connect_AND(
				NewBlock(offset_b, MaskIPv6, hostlist[i+1] , CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(offset_a, MaskIPv6, hostlist[i] , CMP_EQ, FUNC_NONE, NULL )
			);
		block = Connect_OR(block, b);
		i += 2;
	}

	return block;

} // End of ChainHosts

uint64_t VerifyMac(char *s) {
uint64_t mac;
size_t slen = strlen(s);
long l;
char *p, *q, *r;
int i;

	if ( slen > 17 )
		return 0; 

	for (i=0; i<slen; i++ ) {
		if ( !isxdigit(s[i]) && s[i] != ':' ) 
			return 0;
	}

	p = strdup(s);
	if ( !p ) {
		yyerror("malloc() error");
		return 0;
	}

	mac = 0;
	i = 0;	// number of MAC octets must be 6
	r = p;
	q = strchr(r, ':');
	while ( r && i < 6 ) {
		if ( q ) 
			*q = '\0';
		l = strtol(r, NULL, 16);
		if ( (i == 0 && errno == EINVAL) ) {
			free(p);
			return 0;
		}
		if ( l > 255 ) {
			free(p);
			return 0;
		}

		mac = ( mac << 8 ) | (l & 0xFF );
		i++;

		if ( q ) {
			r = ++q;
			q = strchr(r, ':');
		} else 
			r = NULL;
	}

	if ( i != 6 )
		return 0;

	return mac;

} // End of VerifyMac

static int InitSymbols(void) {
int i;

	// already initialised?
	if ( fwd_status )
		return 1;

	// fill fwd status cache table
	fwd_status = ( char **)calloc(256, sizeof(char *));
	if ( !fwd_status ) {
		fprintf(stderr, "malloc(): %s line %d: %s", __FILE__, __LINE__, strerror (errno));
		return 0;
	}
	i=0;
	while ( fwd_status_def_list[i].name ) {
		uint32_t j = fwd_status_def_list[i].id;
		fwd_status[j] = fwd_status_def_list[i].name;
		i++;
	}
	return 1;

} // End of InitSymbols

static uint32_t Get_fwd_status_id(char *status) {
int i;

	if ( !fwd_status && !InitSymbols() )
		yyerror("malloc() error");

	i = 0;
	while ( i < 256 ) {
		if ( fwd_status[i] && strcasecmp(fwd_status[i], status) == 0 ) 
			return i;
		i++;
	}
	return 256;

} // End of Get_fwd_status_id

static char *stripWord(char *word) {
	char *w = strdup(word);

	// strip ", ' and /' from begin/end of string
	if ( w[0] == 0x27 || w[0] == 0x22 || w[0] == 0x2f ) w++;
  size_t last = strlen(w) -1;

	if ( w[last] == 0x27 || w[last] == 0x22 || w[last] == 0x2f ) w[last] = '\0';

	return w;
} // End of stripWord

static int IsMD5(char *string) {

	int i = 0;
	for (i=0; i<32; i++) {
		char c = string[i];
		if ( c == '\0' || !isxdigit(c))
		return 0;
	}
	return string[i] == '\0';

} // End of IsMD5
#line 1115 "grammar.c"

/* For use in generated program */
#define yydepth (int)(yystack.s_mark - yystack.s_base)
#if YYBTYACC
#define yytrial (yyps->save)
#endif /* YYBTYACC */

#if YYDEBUG
#include <stdio.h>	/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    YYINT *newss;
    YYSTYPE *newvs;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    YYLTYPE *newps;
#endif

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return YYENOMEM;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = (int) (data->s_mark - data->s_base);
    newss = (YYINT *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return YYENOMEM;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return YYENOMEM;

    data->l_base = newvs;
    data->l_mark = newvs + i;

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    newps = (YYLTYPE *)realloc(data->p_base, newsize * sizeof(*newps));
    if (newps == 0)
        return YYENOMEM;

    data->p_base = newps;
    data->p_mark = newps + i;
#endif

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;

#if YYDEBUG
    if (yydebug)
        fprintf(stderr, "%sdebug: stack size increased to %d\n", YYPREFIX, newsize);
#endif
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    free(data->p_base);
#endif
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif /* YYPURE || defined(YY_NO_LEAKS) */
#if YYBTYACC

static YYParseState *
yyNewState(unsigned size)
{
    YYParseState *p = (YYParseState *) malloc(sizeof(YYParseState));
    if (p == NULL) return NULL;

    p->yystack.stacksize = size;
    if (size == 0)
    {
        p->yystack.s_base = NULL;
        p->yystack.l_base = NULL;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        p->yystack.p_base = NULL;
#endif
        return p;
    }
    p->yystack.s_base    = (YYINT *) malloc(size * sizeof(YYINT));
    if (p->yystack.s_base == NULL) return NULL;
    p->yystack.l_base    = (YYSTYPE *) malloc(size * sizeof(YYSTYPE));
    if (p->yystack.l_base == NULL) return NULL;
    memset(p->yystack.l_base, 0, size * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    p->yystack.p_base    = (YYLTYPE *) malloc(size * sizeof(YYLTYPE));
    if (p->yystack.p_base == NULL) return NULL;
    memset(p->yystack.p_base, 0, size * sizeof(YYLTYPE));
#endif

    return p;
}

static void
yyFreeState(YYParseState *p)
{
    yyfreestack(&p->yystack);
    free(p);
}
#endif /* YYBTYACC */

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab
#if YYBTYACC
#define YYVALID        do { if (yyps->save)            goto yyvalid; } while(0)
#define YYVALID_NESTED do { if (yyps->save && \
                                yyps->save->save == 0) goto yyvalid; } while(0)
#endif /* YYBTYACC */

int
YYPARSE_DECL()
{
    int yym, yyn, yystate, yyresult;
#if YYBTYACC
    int yynewerrflag;
    YYParseState *yyerrctx = NULL;
#endif /* YYBTYACC */
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    YYLTYPE  yyerror_loc_range[3]; /* position of error start/end (0 unused) */
#endif
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
    if (yydebug)
        fprintf(stderr, "%sdebug[<# of symbols on state stack>]\n", YYPREFIX);
#endif
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    memset(yyerror_loc_range, 0, sizeof(yyerror_loc_range));
#endif

#if YYBTYACC
    yyps = yyNewState(0); if (yyps == 0) goto yyenomem;
    yyps->save = 0;
#endif /* YYBTYACC */
    yym = 0;
    /* yyn is set below */
    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark = yystack.p_base;
#endif
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
#if YYBTYACC
        do {
        if (yylvp < yylve)
        {
            /* we're currently re-reading tokens */
            yylval = *yylvp++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yylloc = *yylpp++;
#endif
            yychar = *yylexp++;
            break;
        }
        if (yyps->save)
        {
            /* in trial mode; save scanner results for future parse attempts */
            if (yylvp == yylvlim)
            {   /* Enlarge lexical value queue */
                size_t p = (size_t) (yylvp - yylvals);
                size_t s = (size_t) (yylvlim - yylvals);

                s += YYLVQUEUEGROWTH;
                if ((yylexemes = (YYINT *)realloc(yylexemes, s * sizeof(YYINT))) == NULL) goto yyenomem;
                if ((yylvals   = (YYSTYPE *)realloc(yylvals, s * sizeof(YYSTYPE))) == NULL) goto yyenomem;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                if ((yylpsns   = (YYLTYPE *)realloc(yylpsns, s * sizeof(YYLTYPE))) == NULL) goto yyenomem;
#endif
                yylvp   = yylve = yylvals + p;
                yylvlim = yylvals + s;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylpp   = yylpe = yylpsns + p;
                yylplim = yylpsns + s;
#endif
                yylexp  = yylexemes + p;
            }
            *yylexp = (YYINT) YYLEX;
            *yylvp++ = yylval;
            yylve++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            *yylpp++ = yylloc;
            yylpe++;
#endif
            yychar = *yylexp++;
            break;
        }
        /* normal operation, no conflict encountered */
#endif /* YYBTYACC */
        yychar = YYLEX;
#if YYBTYACC
        } while (0);
#endif /* YYBTYACC */
        if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            fprintf(stderr, "%s[%d]: state %d, reading token %d (%s)",
                            YYDEBUGSTR, yydepth, yystate, yychar, yys);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
            if (!yytrial)
#endif /* YYBTYACC */
                fprintf(stderr, " <%s>", YYSTYPE_TOSTRING(yychar, yylval));
#endif
            fputc('\n', stderr);
        }
#endif
    }
#if YYBTYACC

    /* Do we have a conflict? */
    if (((yyn = yycindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
        yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        YYINT ctry;

        if (yypath)
        {
            YYParseState *save;
#if YYDEBUG
            if (yydebug)
                fprintf(stderr, "%s[%d]: CONFLICT in state %d: following successful trial parse\n",
                                YYDEBUGSTR, yydepth, yystate);
#endif
            /* Switch to the next conflict context */
            save = yypath;
            yypath = save->save;
            save->save = NULL;
            ctry = save->ctry;
            if (save->state != yystate) YYABORT;
            yyFreeState(save);

        }
        else
        {

            /* Unresolved conflict - start/continue trial parse */
            YYParseState *save;
#if YYDEBUG
            if (yydebug)
            {
                fprintf(stderr, "%s[%d]: CONFLICT in state %d. ", YYDEBUGSTR, yydepth, yystate);
                if (yyps->save)
                    fputs("ALREADY in conflict, continuing trial parse.\n", stderr);
                else
                    fputs("Starting trial parse.\n", stderr);
            }
#endif
            save                  = yyNewState((unsigned)(yystack.s_mark - yystack.s_base + 1));
            if (save == NULL) goto yyenomem;
            save->save            = yyps->save;
            save->state           = yystate;
            save->errflag         = yyerrflag;
            save->yystack.s_mark  = save->yystack.s_base + (yystack.s_mark - yystack.s_base);
            memcpy (save->yystack.s_base, yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
            save->yystack.l_mark  = save->yystack.l_base + (yystack.l_mark - yystack.l_base);
            memcpy (save->yystack.l_base, yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            save->yystack.p_mark  = save->yystack.p_base + (yystack.p_mark - yystack.p_base);
            memcpy (save->yystack.p_base, yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
            ctry                  = yytable[yyn];
            if (yyctable[ctry] == -1)
            {
#if YYDEBUG
                if (yydebug && yychar >= YYEOF)
                    fprintf(stderr, "%s[%d]: backtracking 1 token\n", YYDEBUGSTR, yydepth);
#endif
                ctry++;
            }
            save->ctry = ctry;
            if (yyps->save == NULL)
            {
                /* If this is a first conflict in the stack, start saving lexemes */
                if (!yylexemes)
                {
                    yylexemes = (YYINT *) malloc((YYLVQUEUEGROWTH) * sizeof(YYINT));
                    if (yylexemes == NULL) goto yyenomem;
                    yylvals   = (YYSTYPE *) malloc((YYLVQUEUEGROWTH) * sizeof(YYSTYPE));
                    if (yylvals == NULL) goto yyenomem;
                    yylvlim   = yylvals + YYLVQUEUEGROWTH;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    yylpsns   = (YYLTYPE *) malloc((YYLVQUEUEGROWTH) * sizeof(YYLTYPE));
                    if (yylpsns == NULL) goto yyenomem;
                    yylplim   = yylpsns + YYLVQUEUEGROWTH;
#endif
                }
                if (yylvp == yylve)
                {
                    yylvp  = yylve = yylvals;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    yylpp  = yylpe = yylpsns;
#endif
                    yylexp = yylexemes;
                    if (yychar >= YYEOF)
                    {
                        *yylve++ = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                        *yylpe++ = yylloc;
#endif
                        *yylexp  = (YYINT) yychar;
                        yychar   = YYEMPTY;
                    }
                }
            }
            if (yychar >= YYEOF)
            {
                yylvp--;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylpp--;
#endif
                yylexp--;
                yychar = YYEMPTY;
            }
            save->lexeme = (int) (yylvp - yylvals);
            yyps->save   = save;
        }
        if (yytable[yyn] == ctry)
        {
#if YYDEBUG
            if (yydebug)
                fprintf(stderr, "%s[%d]: state %d, shifting to state %d\n",
                                YYDEBUGSTR, yydepth, yystate, yyctable[ctry]);
#endif
            if (yychar < 0)
            {
                yylvp++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylpp++;
#endif
                yylexp++;
            }
            if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM)
                goto yyoverflow;
            yystate = yyctable[ctry];
            *++yystack.s_mark = (YYINT) yystate;
            *++yystack.l_mark = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            *++yystack.p_mark = yylloc;
#endif
            yychar  = YYEMPTY;
            if (yyerrflag > 0) --yyerrflag;
            goto yyloop;
        }
        else
        {
            yyn = yyctable[ctry];
            goto yyreduce;
        }
    } /* End of code dealing with conflicts */
#endif /* YYBTYACC */
    if (((yyn = yysindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
#if YYDEBUG
        if (yydebug)
            fprintf(stderr, "%s[%d]: state %d, shifting to state %d\n",
                            YYDEBUGSTR, yydepth, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        *++yystack.p_mark = yylloc;
#endif
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if (((yyn = yyrindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag != 0) goto yyinrecovery;
#if YYBTYACC

    yynewerrflag = 1;
    goto yyerrhandler;
    goto yyerrlab; /* redundant goto avoids 'unused label' warning */

yyerrlab:
    /* explicit YYERROR from an action -- pop the rhs of the rule reduced
     * before looking for error recovery */
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark -= yym;
#endif

    yynewerrflag = 0;
yyerrhandler:
    while (yyps->save)
    {
        int ctry;
        YYParseState *save = yyps->save;
#if YYDEBUG
        if (yydebug)
            fprintf(stderr, "%s[%d]: ERROR in state %d, CONFLICT BACKTRACKING to state %d, %d tokens\n",
                            YYDEBUGSTR, yydepth, yystate, yyps->save->state,
                    (int)(yylvp - yylvals - yyps->save->lexeme));
#endif
        /* Memorize most forward-looking error state in case it's really an error. */
        if (yyerrctx == NULL || yyerrctx->lexeme < yylvp - yylvals)
        {
            /* Free old saved error context state */
            if (yyerrctx) yyFreeState(yyerrctx);
            /* Create and fill out new saved error context state */
            yyerrctx                 = yyNewState((unsigned)(yystack.s_mark - yystack.s_base + 1));
            if (yyerrctx == NULL) goto yyenomem;
            yyerrctx->save           = yyps->save;
            yyerrctx->state          = yystate;
            yyerrctx->errflag        = yyerrflag;
            yyerrctx->yystack.s_mark = yyerrctx->yystack.s_base + (yystack.s_mark - yystack.s_base);
            memcpy (yyerrctx->yystack.s_base, yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
            yyerrctx->yystack.l_mark = yyerrctx->yystack.l_base + (yystack.l_mark - yystack.l_base);
            memcpy (yyerrctx->yystack.l_base, yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yyerrctx->yystack.p_mark = yyerrctx->yystack.p_base + (yystack.p_mark - yystack.p_base);
            memcpy (yyerrctx->yystack.p_base, yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
            yyerrctx->lexeme         = (int) (yylvp - yylvals);
        }
        yylvp          = yylvals   + save->lexeme;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        yylpp          = yylpsns   + save->lexeme;
#endif
        yylexp         = yylexemes + save->lexeme;
        yychar         = YYEMPTY;
        yystack.s_mark = yystack.s_base + (save->yystack.s_mark - save->yystack.s_base);
        memcpy (yystack.s_base, save->yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
        yystack.l_mark = yystack.l_base + (save->yystack.l_mark - save->yystack.l_base);
        memcpy (yystack.l_base, save->yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        yystack.p_mark = yystack.p_base + (save->yystack.p_mark - save->yystack.p_base);
        memcpy (yystack.p_base, save->yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
        ctry           = ++save->ctry;
        yystate        = save->state;
        /* We tried shift, try reduce now */
        if ((yyn = yyctable[ctry]) >= 0) goto yyreduce;
        yyps->save     = save->save;
        save->save     = NULL;
        yyFreeState(save);

        /* Nothing left on the stack -- error */
        if (!yyps->save)
        {
#if YYDEBUG
            if (yydebug)
                fprintf(stderr, "%sdebug[%d,trial]: trial parse FAILED, entering ERROR mode\n",
                                YYPREFIX, yydepth);
#endif
            /* Restore state as it was in the most forward-advanced error */
            yylvp          = yylvals   + yyerrctx->lexeme;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yylpp          = yylpsns   + yyerrctx->lexeme;
#endif
            yylexp         = yylexemes + yyerrctx->lexeme;
            yychar         = yylexp[-1];
            yylval         = yylvp[-1];
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yylloc         = yylpp[-1];
#endif
            yystack.s_mark = yystack.s_base + (yyerrctx->yystack.s_mark - yyerrctx->yystack.s_base);
            memcpy (yystack.s_base, yyerrctx->yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
            yystack.l_mark = yystack.l_base + (yyerrctx->yystack.l_mark - yyerrctx->yystack.l_base);
            memcpy (yystack.l_base, yyerrctx->yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yystack.p_mark = yystack.p_base + (yyerrctx->yystack.p_mark - yyerrctx->yystack.p_base);
            memcpy (yystack.p_base, yyerrctx->yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
            yystate        = yyerrctx->state;
            yyFreeState(yyerrctx);
            yyerrctx       = NULL;
        }
        yynewerrflag = 1;
    }
    if (yynewerrflag == 0) goto yyinrecovery;
#endif /* YYBTYACC */

    YYERROR_CALL("syntax error");
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yyerror_loc_range[1] = yylloc; /* lookahead position is error start position */
#endif

#if !YYBTYACC
    goto yyerrlab; /* redundant goto avoids 'unused label' warning */
yyerrlab:
#endif
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if (((yyn = yysindex[*yystack.s_mark]) != 0) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    fprintf(stderr, "%s[%d]: state %d, error recovery shifting to state %d\n",
                                    YYDEBUGSTR, yydepth, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                /* lookahead position is error end position */
                yyerror_loc_range[2] = yylloc;
                YYLLOC_DEFAULT(yyloc, yyerror_loc_range, 2); /* position of error span */
                *++yystack.p_mark = yyloc;
#endif
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    fprintf(stderr, "%s[%d]: error recovery discarding state %d\n",
                                    YYDEBUGSTR, yydepth, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                /* the current TOS position is the error start position */
                yyerror_loc_range[1] = *yystack.p_mark;
#endif
#if defined(YYDESTRUCT_CALL)
#if YYBTYACC
                if (!yytrial)
#endif /* YYBTYACC */
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    YYDESTRUCT_CALL("error: discarding state",
                                    yystos[*yystack.s_mark], yystack.l_mark, yystack.p_mark);
#else
                    YYDESTRUCT_CALL("error: discarding state",
                                    yystos[*yystack.s_mark], yystack.l_mark);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
#endif /* defined(YYDESTRUCT_CALL) */
                --yystack.s_mark;
                --yystack.l_mark;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                --yystack.p_mark;
#endif
            }
        }
    }
    else
    {
        if (yychar == YYEOF) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            fprintf(stderr, "%s[%d]: state %d, error recovery discarding token %d (%s)\n",
                            YYDEBUGSTR, yydepth, yystate, yychar, yys);
        }
#endif
#if defined(YYDESTRUCT_CALL)
#if YYBTYACC
        if (!yytrial)
#endif /* YYBTYACC */
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            YYDESTRUCT_CALL("error: discarding token", yychar, &yylval, &yylloc);
#else
            YYDESTRUCT_CALL("error: discarding token", yychar, &yylval);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
#endif /* defined(YYDESTRUCT_CALL) */
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
    yym = yylen[yyn];
#if YYDEBUG
    if (yydebug)
    {
        fprintf(stderr, "%s[%d]: state %d, reducing by rule %d (%s)",
                        YYDEBUGSTR, yydepth, yystate, yyn, yyrule[yyn]);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
        if (!yytrial)
#endif /* YYBTYACC */
            if (yym > 0)
            {
                int i;
                fputc('<', stderr);
                for (i = yym; i > 0; i--)
                {
                    if (i != yym) fputs(", ", stderr);
                    fputs(YYSTYPE_TOSTRING(yystos[yystack.s_mark[1-i]],
                                           yystack.l_mark[1-i]), stderr);
                }
                fputc('>', stderr);
            }
#endif
        fputc('\n', stderr);
    }
#endif
    if (yym > 0)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)

    /* Perform position reduction */
    memset(&yyloc, 0, sizeof(yyloc));
#if YYBTYACC
    if (!yytrial)
#endif /* YYBTYACC */
    {
        YYLLOC_DEFAULT(yyloc, &yystack.p_mark[-yym], yym);
        /* just in case YYERROR is invoked within the action, save
           the start of the rhs as the error start position */
        yyerror_loc_range[1] = yystack.p_mark[1-yym];
    }
#endif

    switch (yyn)
    {
case 2:
#line 162 "grammar.y"
	{   
		StartNode = yystack.l_mark[0].value; 
	}
#line 1790 "grammar.c"
break;
case 3:
#line 167 "grammar.y"
	{ /* this is an unconditionally true expression, as a filter applies in any case */
		yyval.param.self = NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, (void *)-1 ); 
	}
#line 1797 "grammar.c"
break;
case 4:
#line 171 "grammar.y"
	{	
		if ( !ScreenIdentString(yystack.l_mark[0].s) ) {
			yyerror("Illegal ident string");
			YYABORT;
		}

		uint32_t	index = AddIdent(yystack.l_mark[0].s);
		yyval.param.self = NewBlock(0, 0, index, CMP_IDENT, FUNC_NONE, NULL ); 
	}
#line 1810 "grammar.c"
break;
case 5:
#line 181 "grammar.y"
	{ 
		yyval.param.self = NewBlock(OffsetRecordMFlags, (1LL << ShiftRecordMFlags)  & MaskRecordMFlags, 
					(0LL << ShiftRecordMFlags)  & MaskRecordMFlags, CMP_EQ, FUNC_NONE, NULL); 
	}
#line 1818 "grammar.c"
break;
case 6:
#line 186 "grammar.y"
	{ 
		yyval.param.self = NewBlock(OffsetRecordMFlags, (1LL << ShiftRecordMFlags)  & MaskRecordMFlags, 
					(1LL << ShiftRecordMFlags)  & MaskRecordMFlags, CMP_EQ, FUNC_NONE, NULL); 
	}
#line 1826 "grammar.c"
break;
case 7:
#line 191 "grammar.y"
	{ 
		int64_t	proto;
		proto = yystack.l_mark[0].value;

		if ( proto > 255 ) {
			yyerror("Protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetProto, MaskProto, (proto << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL); 

	}
#line 1845 "grammar.c"
break;
case 8:
#line 207 "grammar.y"
	{ 
		int64_t	proto;
		proto = ProtoNum(yystack.l_mark[0].s);

		if ( proto > 255 ) {
			yyerror("Protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetProto, MaskProto, (proto << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL); 
	}
#line 1863 "grammar.c"
break;
case 9:
#line 222 "grammar.y"
	{ 
		int64_t	proto;
		proto = yystack.l_mark[0].value;

		if ( proto > 255 ) {
			yyerror("Tunnel protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown tunnel protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetTUNPROTO, MaskTUNPROTO, (proto << ShiftTUNPROTO)  & MaskTUNPROTO, CMP_EQ, FUNC_NONE, NULL); 

	}
#line 1882 "grammar.c"
break;
case 10:
#line 238 "grammar.y"
	{ 
		int64_t	proto;
		proto = ProtoNum(yystack.l_mark[0].s);

		if ( proto > 255 ) {
			yyerror("Tunnel protocol number > 255");
			YYABORT;
		}
		if ( proto < 0 ) {
			yyerror("Unknown tunnel protocol");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetTUNPROTO, MaskTUNPROTO, (proto << ShiftTUNPROTO)  & MaskTUNPROTO, CMP_EQ, FUNC_NONE, NULL); 
	}
#line 1900 "grammar.c"
break;
case 11:
#line 254 "grammar.y"
	{ 

		switch ( yystack.l_mark[-3].param.direction ) {
			case DIR_UNSPEC:
			case DIR_IN: 
				yyval.param.self = NewBlock(OffsetPackets, MaskPackets, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case DIR_OUT: 
				yyval.param.self = NewBlock(OffsetOutPackets, MaskPackets, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			default:
				/* should never happen */
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 1921 "grammar.c"
break;
case 12:
#line 272 "grammar.y"
	{	

		switch ( yystack.l_mark[-3].param.direction ) {
			case DIR_UNSPEC:
			case DIR_IN: 
				yyval.param.self = NewBlock(OffsetBytes, MaskBytes, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case DIR_OUT: 
				yyval.param.self = NewBlock(OffsetOutBytes, MaskBytes, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 1941 "grammar.c"
break;
case 13:
#line 289 "grammar.y"
	{	
			yyval.param.self = NewBlock(OffsetAggrFlows, MaskFlows, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
#line 1948 "grammar.c"
break;
case 14:
#line 293 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_PPS, NULL); 
	}
#line 1955 "grammar.c"
break;
case 15:
#line 297 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_BPS, NULL); 
	}
#line 1962 "grammar.c"
break;
case 16:
#line 301 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_BPP, NULL); 
	}
#line 1969 "grammar.c"
break;
case 17:
#line 305 "grammar.y"
	{	
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_DURATION, NULL); 
	}
#line 1976 "grammar.c"
break;
case 18:
#line 309 "grammar.y"
	{	
		yyval.param.self = NewBlock(OffsetFlowCount, MaskFlowCount, (yystack.l_mark[0].value << ShiftFlowCount) & MaskFlowCount, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
#line 1983 "grammar.c"
break;
case 19:
#line 313 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("TOS must be 0..255");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case DIR_UNSPEC:
			case SOURCE:
				yyval.param.self = NewBlock(OffsetTos, MaskTos, (yystack.l_mark[0].value << ShiftTos) & MaskTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetDstTos, MaskDstTos, (yystack.l_mark[0].value << ShiftDstTos) & MaskDstTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
				break;
			case SOURCE_OR_DESTINATION: 
				yyval.param.self = Connect_OR(
					NewBlock(OffsetTos, MaskTos, (yystack.l_mark[0].value << ShiftTos) & MaskTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL),
					NewBlock(OffsetDstTos, MaskDstTos, (yystack.l_mark[0].value << ShiftDstTos) & MaskDstTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetTos, MaskTos, (yystack.l_mark[0].value << ShiftTos) & MaskTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL),
					NewBlock(OffsetDstTos, MaskDstTos, (yystack.l_mark[0].value << ShiftDstTos) & MaskDstTos, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
			}
	}
#line 2018 "grammar.c"
break;
case 20:
#line 345 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 10 ) {
			yyerror("Netflow version must be <= 10");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetRecordVersion, MaskRecordVersion, (yystack.l_mark[0].value << ShiftRecordVersion) & MaskRecordVersion, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
#line 2029 "grammar.c"
break;
case 21:
#line 353 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 63 ) {
			yyerror("Flags must be 0..63");
			YYABORT;
		}
		yyval.param.self = Connect_AND(
			/* imply flags with proto TCP*/
			NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_TCP << ShiftProto) & MaskProto, CMP_EQ, FUNC_NONE, NULL),
			NewBlock(OffsetFlags, MaskFlags, (yystack.l_mark[0].value << ShiftFlags) & MaskFlags, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
		);
	}
#line 2044 "grammar.c"
break;
case 22:
#line 365 "grammar.y"
	{	
		/* handle special case with 'AS' takes as flags. and not AS number*/
		uint64_t fl = 0;
		fl |= 16;
		fl |= 2;
		yyval.param.self = Connect_AND(
			/* imply flags with proto TCP */
			NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_TCP << ShiftProto) & MaskProto, CMP_EQ, FUNC_NONE, NULL),
			NewBlock(OffsetFlags, (fl << ShiftFlags) & MaskFlags, (fl << ShiftFlags) & MaskFlags, CMP_FLAGS, FUNC_NONE, NULL)
		);
	}
#line 2059 "grammar.c"
break;
case 23:
#line 377 "grammar.y"
	{	
		uint64_t fl = 0;
		int cnt     = 0;
		size_t		len = strlen(yystack.l_mark[0].s);

		if ( len > 7 ) {
			yyerror("Too many flags");
			YYABORT;
		}

		if ( strchr(yystack.l_mark[0].s, 'F') ) { fl |=  1; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'S') ) { fl |=  2; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'R') ) { fl |=  4; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'P') ) { fl |=  8; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'A') ) { fl |=  16; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'U') ) { fl |=  32; cnt++; }
		if ( strchr(yystack.l_mark[0].s, 'X') ) { fl =  63; cnt++; }

		if ( cnt != len ) {
			yyerror("Too many flags");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			/* imply flags with proto TCP*/
			NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_TCP << ShiftProto) & MaskProto, CMP_EQ, FUNC_NONE, NULL),
			NewBlock(OffsetFlags, (fl << ShiftFlags) & MaskFlags, (fl << ShiftFlags) & MaskFlags, CMP_FLAGS, FUNC_NONE, NULL)
		);
	}
#line 2092 "grammar.c"
break;
case 24:
#line 407 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		if ( ret == -2 ) {
			/* could not resolv host => 'not any'*/
			yyval.param.self = Invert(NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, NULL )); 
		} else {
			uint64_t offsets[4] = {OffsetSrcIPv6a, OffsetSrcIPv6b, OffsetDstIPv6a, OffsetDstIPv6b };
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			switch ( yystack.l_mark[-2].param.direction ) {
				case SOURCE:
				case DESTINATION:
					yyval.param.self = ChainHosts(offsets, IPstack, num_ip, yystack.l_mark[-2].param.direction);
					break;
				case DIR_UNSPEC:
				case SOURCE_OR_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_OR(src, dst);
					} break;
				case SOURCE_AND_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_AND(src, dst);
					} break;
				default:
					yyerror("This token is not expected here!");
					YYABORT;
	
			} /* End of switch*/

		}
	}
#line 2141 "grammar.c"
break;
case 25:
#line 453 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		if ( ret == -2 ) {
			/* could not resolv host => 'not any'*/
			yyval.param.self = Invert(NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, NULL )); 
		} else {
			uint64_t offsets[4] = {OffsetTUNSRCIP, OffsetTUNSRCIP+1, OffsetTUNDSTIP, OffsetTUNDSTIP+1 };
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			switch ( yystack.l_mark[-2].param.direction ) {
				case SOURCE:
				case DESTINATION:
					yyval.param.self = ChainHosts(offsets, IPstack, num_ip, yystack.l_mark[-2].param.direction);
					break;
				case DIR_UNSPEC:
				case SOURCE_OR_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_OR(src, dst);
					} break;
				case SOURCE_AND_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_AND(src, dst);
					} break;
				default:
					yyerror("This token is not expected here!");
					YYABORT;
	
			} /* End of switch*/

		}
	}
#line 2190 "grammar.c"
break;
case 26:
#line 499 "grammar.y"
	{ 	

		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetSrcIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetDstIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetSrcIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetDstIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetSrcIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetDstIPv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		}
	}
#line 2221 "grammar.c"
break;
case 27:
#line 527 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			NewBlock(OffsetNexthopv6b, MaskIPv6, IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetNexthopv6a, MaskIPv6, IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
		);
	}
#line 2251 "grammar.c"
break;
case 28:
#line 554 "grammar.y"
	{ 	

		yyval.param.self = NewBlock(OffsetNexthopv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );

	}
#line 2260 "grammar.c"
break;
case 29:
#line 560 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			NewBlock(OffsetBGPNexthopv6b, MaskIPv6, IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetBGPNexthopv6a, MaskIPv6, IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
		);
	}
#line 2290 "grammar.c"
break;
case 30:
#line 587 "grammar.y"
	{ 	
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		yyval.param.self = Connect_AND(
			NewBlock(OffsetRouterv6b, MaskIPv6, IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetRouterv6a, MaskIPv6, IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
		);
	}
#line 2320 "grammar.c"
break;
case 31:
#line 614 "grammar.y"
	{ 	
		yyval.param.self = NewBlock(OffsetClientLatency, MaskLatency, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
#line 2327 "grammar.c"
break;
case 32:
#line 618 "grammar.y"
	{ 	
		yyval.param.self = NewBlock(OffsetServerLatency, MaskLatency, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
#line 2334 "grammar.c"
break;
case 33:
#line 622 "grammar.y"
	{ 	
		yyval.param.self = NewBlock(OffsetAppLatency, MaskLatency, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL); 
	}
#line 2341 "grammar.c"
break;
case 34:
#line 626 "grammar.y"
	{ 	
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Router SysID expected between be 1..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetExporterSysID, MaskExporterSysID, (yystack.l_mark[0].value << ShiftExporterSysID) & MaskExporterSysID, CMP_EQ, FUNC_NONE, NULL); 
	}
#line 2352 "grammar.c"
break;
case 35:
#line 634 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Port outside of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetPort, MaskSrcPort, (yystack.l_mark[0].value << ShiftSrcPort) & MaskSrcPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetPort, MaskDstPort, (yystack.l_mark[0].value << ShiftDstPort) & MaskDstPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetPort, MaskSrcPort, (yystack.l_mark[0].value << ShiftSrcPort) & MaskSrcPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetPort, MaskDstPort, (yystack.l_mark[0].value << ShiftDstPort) & MaskDstPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetPort, MaskSrcPort, (yystack.l_mark[0].value << ShiftSrcPort) & MaskSrcPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetPort, MaskDstPort, (yystack.l_mark[0].value << ShiftDstPort) & MaskDstPort, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/

	}
#line 2388 "grammar.c"
break;
case 36:
#line 667 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 0xFFFFFFFFLL ) {
			yyerror("Invalid ingress vrf ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetIVRFID, MaskIVRFID, ( yystack.l_mark[0].value << ShiftIVRFID) & MaskIVRFID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
	}
#line 2399 "grammar.c"
break;
case 37:
#line 675 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 0xFFFFFFFFLL ) {
			yyerror("Invalid egress vrf ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetEVRFID, MaskEVRFID, ( yystack.l_mark[0].value << ShiftEVRFID) & MaskEVRFID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
	}
#line 2410 "grammar.c"
break;
case 38:
#line 683 "grammar.y"
	{
			uint64_t index = pfActionNr(yystack.l_mark[0].s);
			if ( index < 0 ) {
				yyerror("Invalid pf action");
				printf("Possible pf action values: ");
				pfListActions();
			} else {
				yyval.param.self = NewBlock(OffsetPfInfo, MaskPfAction, ( index << ShiftPfAction) & MaskPfAction, CMP_EQ, FUNC_NONE, NULL );
			}
}
#line 2424 "grammar.c"
break;
case 39:
#line 694 "grammar.y"
	{
			uint64_t index = pfActionNr("nat");
			if ( index < 0 ) {
				yyerror("Invalid pf action");
				printf("Possible pf action values: ");
				pfListActions();
			} else {
				yyval.param.self = NewBlock(OffsetPfInfo, MaskPfAction, ( index << ShiftPfAction) & MaskPfAction, CMP_EQ, FUNC_NONE, NULL );
			}
}
#line 2438 "grammar.c"
break;
case 40:
#line 705 "grammar.y"
	{
			uint64_t index = pfReasonNr(yystack.l_mark[0].s);
			if ( index < 0 ) {
				yyerror("Invalid pf reason");
				printf("Possible pf reason values: ");
				pfListReasons();
			} else {
				yyval.param.self = NewBlock(OffsetPfInfo, MaskPfReason, ( index << ShiftPfReason) & MaskPfReason, CMP_EQ, FUNC_NONE, NULL );
			}
}
#line 2452 "grammar.c"
break;
case 41:
#line 716 "grammar.y"
	{
	union {
		char ifName[16];
		uint64_t val[2];
	} ifValue = {0};
	size_t len = strlen(yystack.l_mark[0].s);
	if ( len > 15 ) {
				yyerror("Invalid pf interface name length");
	}
	memcpy(ifValue.ifName, yystack.l_mark[0].s, len);
	yyval.param.self = Connect_AND(
					NewBlock(OffsetPfIfname, MaskPfIfname, ifValue.val[0], CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetPfIfname+1, MaskPfIfname, ifValue.val[1], CMP_EQ, FUNC_NONE, NULL )
				);
}
#line 2471 "grammar.c"
break;
case 42:
#line 732 "grammar.y"
	{
	yyval.param.self = NewBlock(OffsetPfInfo, MaskPfRulenr, ( yystack.l_mark[0].value << ShiftPfRulenr) & MaskPfRulenr, CMP_EQ, FUNC_NONE, NULL );
}
#line 2478 "grammar.c"
break;
case 43:
#line 736 "grammar.y"
	{
	yyval.param.self = NewBlock(OffsetPfInfo, MaskPfDir, ( 1LL << ShiftPfDir) & MaskPfDir, CMP_EQ, FUNC_NONE, NULL );
}
#line 2485 "grammar.c"
break;
case 44:
#line 740 "grammar.y"
	{
	yyval.param.self = NewBlock(OffsetPfInfo, MaskPfDir, ( 0LL << ShiftPfDir) & MaskPfDir, CMP_EQ, FUNC_NONE, NULL );
}
#line 2492 "grammar.c"
break;
case 45:
#line 744 "grammar.y"
	{	
#ifdef NSEL
		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
					yyval.param.self = NewBlock(OffsetPort, MaskSrcPort, ShiftSrcPort, CMP_EQ, FUNC_PBLOCK, NULL );
				break;
			case DESTINATION:
					yyval.param.self = NewBlock(OffsetPort, MaskDstPort, ShiftDstPort, CMP_EQ, FUNC_PBLOCK, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetPort, MaskSrcPort, ShiftSrcPort, CMP_EQ, FUNC_PBLOCK, NULL ),
					NewBlock(OffsetPort, MaskDstPort, ShiftDstPort, CMP_EQ, FUNC_PBLOCK, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/

#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 2522 "grammar.c"
break;
case 46:
#line 771 "grammar.y"
	{ 	
		struct ULongListNode *node;
		ULongtree_t *root = NULL;

		if ( yystack.l_mark[-5].param.direction == DIR_UNSPEC || yystack.l_mark[-5].param.direction == SOURCE_OR_DESTINATION || yystack.l_mark[-5].param.direction == SOURCE_AND_DESTINATION ) {
			/* src and/or dst port*/
			/* we need a second rbtree due to different shifts for src and dst ports*/
			root = malloc(sizeof(ULongtree_t));

			struct ULongListNode *n;
			if ( root == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}
			RB_INIT(root);

			RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
				if ( node->value > 65535 ) {
					yyerror("Port outside of range 0..65535");
					YYABORT;
				}
				if ((n = malloc(sizeof(struct ULongListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				n->value 	= (node->value << ShiftDstPort) & MaskDstPort;
				node->value = (node->value << ShiftSrcPort) & MaskSrcPort;
				RB_INSERT(ULongtree, root, n);
			}
		}

		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftSrcPort) & MaskSrcPort;
				}
				yyval.param.self = NewBlock(OffsetPort, MaskSrcPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftDstPort) & MaskDstPort;
				}
				yyval.param.self = NewBlock(OffsetPort, MaskDstPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetPort, MaskSrcPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetPort, MaskDstPort, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetPort, MaskSrcPort, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetPort, MaskDstPort, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/
	}
#line 2588 "grammar.c"
break;
case 47:
#line 834 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("ICMP type of range 0..255");
			YYABORT;
		}
		yyval.param.self = Connect_AND(
			/* imply ICMP-TYPE with a proto ICMP block*/
			Connect_OR (
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMP << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL), 
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMPV6 << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL)
			),
			NewBlock(OffsetICMP, MaskICMPtype, (yystack.l_mark[0].value << ShiftICMPtype) & MaskICMPtype, CMP_EQ, FUNC_NONE, NULL )
		);
	}
#line 2606 "grammar.c"
break;
case 48:
#line 849 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("ICMP code of range 0..255");
			YYABORT;
		}
		yyval.param.self = Connect_AND(
			/* imply ICMP-CODE with a proto ICMP block*/
			Connect_OR (
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMP << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL), 
				NewBlock(OffsetProto, MaskProto, ((uint64_t)IPPROTO_ICMPV6 << ShiftProto)  & MaskProto, CMP_EQ, FUNC_NONE, NULL)
			),
			NewBlock(OffsetICMP, MaskICMPcode, (yystack.l_mark[0].value << ShiftICMPcode) & MaskICMPcode, CMP_EQ, FUNC_NONE, NULL )
		);
	}
#line 2624 "grammar.c"
break;
case 49:
#line 864 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Engine type of range 0..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetRouterID, MaskEngineType, (yystack.l_mark[0].value << ShiftEngineType) & MaskEngineType, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);
	}
#line 2635 "grammar.c"
break;
case 50:
#line 872 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Engine ID of range 0..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetRouterID, MaskEngineID, (yystack.l_mark[0].value << ShiftEngineID) & MaskEngineID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);
	}
#line 2646 "grammar.c"
break;
case 51:
#line 880 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Exporter ID of range 0..65535");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetExporterSysID, MaskExporterSysID, (yystack.l_mark[0].value << ShiftExporterSysID) & MaskExporterSysID, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);
	}
#line 2657 "grammar.c"
break;
case 52:
#line 889 "grammar.y"
	{	
		if ( strlen(yystack.l_mark[0].s) > 16 ) {
			yyerror("Size flowlabel of range 1..16");
			YYABORT;
		}

		yyval.param.self = NewBlock(0, 0, 0, CMP_FLOWLABEL, FUNC_NONE, (void *)strdup(yystack.l_mark[0].s)); 
	}
#line 2669 "grammar.c"
break;
case 53:
#line 898 "grammar.y"
	{
#ifdef NSEL
		if ( strncasecmp(yystack.l_mark[0].s,"ignore", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_IGNORE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"create", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_CREATE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"term", 4) == 0 || strncasecmp(yystack.l_mark[0].s,"delete", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DELETE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if  (strncasecmp(yystack.l_mark[0].s,"deny", 4) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DENIED << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if  (strncasecmp(yystack.l_mark[0].s,"alert", 5) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_ALERT << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else if  (strncasecmp(yystack.l_mark[0].s,"update", 6) == 0) {
			yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_UPDATE << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL );
		} else {
			yyerror("Unknown asa event");
			YYABORT;
		}
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2696 "grammar.c"
break;
case 54:
#line 922 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Invalid xevent ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetConnID, MaskFWevent, ( yystack.l_mark[0].value << ShiftFWevent) & MaskFWevent, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2712 "grammar.c"
break;
case 55:
#line 935 "grammar.y"
	{
#ifdef NSEL
		uint64_t xevent = 0;
		if ( yystack.l_mark[0].param.inout == INGRESS ) {
			xevent = 1001;
		} else if ( yystack.l_mark[0].param.inout == EGRESS ) {
			xevent = 1002;
		} else {
				yyerror("Invalid inout token");
				YYABORT;
		}
		yyval.param.self = Connect_AND(
			NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DENIED << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetConnID, MaskFWXevent, ( xevent << ShiftFWXevent) & MaskFWXevent, CMP_EQ, FUNC_NONE, NULL )
		);
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2736 "grammar.c"
break;
case 56:
#line 955 "grammar.y"
	{
#ifdef NSEL
		uint64_t xevent = 0;
		if( strncasecmp(yystack.l_mark[0].s,"interface", 9) == 0) {
			xevent = 1003;
		} else if( strncasecmp(yystack.l_mark[0].s,"nosyn", 5) == 0) {
			xevent = 1004;
		} else {
			xevent = (uint64_t)strtol(yystack.l_mark[0].s, (char **)NULL, 10);
			if ( (xevent == 0 && errno == EINVAL) || xevent > 65535 ) {
				yyerror("Invalid xevent ID");
				YYABORT;
			}
		}
		yyval.param.self = Connect_AND(
			NewBlock(OffsetConnID, MaskFWevent, ( NSEL_EVENT_DENIED << ShiftFWevent) & MaskFWevent, CMP_EQ, FUNC_NONE, NULL ),
			NewBlock(OffsetConnID, MaskFWXevent, ( xevent << ShiftFWXevent) & MaskFWXevent, CMP_EQ, FUNC_NONE, NULL )
		);
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2763 "grammar.c"
break;
case 57:
#line 979 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Invalid xevent ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetConnID, MaskFWXevent, ( yystack.l_mark[0].value << ShiftFWXevent) & MaskFWXevent, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2779 "grammar.c"
break;
case 58:
#line 992 "grammar.y"
	{ 
#ifdef NSEL
		int af, bytes, ret;
		uint64_t	mask[2];

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);
		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/


		if ( yystack.l_mark[0].value > (bytes*8) ) {
			yyerror("Too many netbits for this IP address");
			YYABORT;
		}

		if ( af == PF_INET ) {
			mask[0] = 0xffffffffffffffffLL;
			mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
		} else {	/* PF_INET6*/
			if ( yystack.l_mark[0].value > 64 ) {
				mask[0] = 0xffffffffffffffffLL;
				mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
			} else {
				mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
				mask[1] = 0;
			}
		}
		/* IP aadresses are stored in network representation */
		mask[0]	 = mask[0];
		mask[1]	 = mask[1];

		IPstack[0] &= mask[0];
		IPstack[1] &= mask[1];

		switch ( yystack.l_mark[-4].param.direction ) {
			case SOURCE:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATESRCv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATESRCv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATEDSTv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEDSTv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					Connect_AND(
						NewBlock(OffsetXLATESRCv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATESRCv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetXLATEDSTv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATEDSTv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					Connect_AND(
						NewBlock(OffsetXLATESRCv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATESRCv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetXLATEDSTv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetXLATEDSTv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2872 "grammar.c"
break;
case 59:
#line 1082 "grammar.y"
	{	
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Port outside of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif

	}
#line 2913 "grammar.c"
break;
case 60:
#line 1120 "grammar.y"
	{
#ifdef NSEL
		uint64_t offset, mask, shift;
		if ( yystack.l_mark[-3].param.inout == INGRESS ) {
			switch (yystack.l_mark[-2].param.acl) {
				case ACL:
					offset = OffsetIngressAclId;
					mask   = MaskIngressAclId;	
					shift  = ShiftIngressAclId;
					break;
				case ACE:
					offset = OffsetIngressAceId;
					mask   = MaskIngressAceId;	
					shift  = ShiftIngressAceId;
					break;
				case XACE:
					offset = OffsetIngressGrpId;
					mask   = MaskIngressGrpId;	
					shift  = ShiftIngressGrpId;
					break;
				default:
					yyerror("Invalid ACL specifier");
					YYABORT;
			}
		} else if ( yystack.l_mark[-3].param.inout == EGRESS && yyval.param.acl == ACL ) {
			offset = OffsetEgressAclId;
			mask   = MaskEgressAclId;	
			shift  = ShiftEgressAclId;
		} else {
			yyerror("ingress/egress syntax error");
			YYABORT;
		}
		yyval.param.self = NewBlock(offset, mask, (yystack.l_mark[0].value << shift) & mask , yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );

#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 2956 "grammar.c"
break;
case 61:
#line 1160 "grammar.y"
	{
#ifdef NSEL
		if ( strncasecmp(yystack.l_mark[0].s,"invalid", 7) == 0) {
			yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( NEL_EVENT_INVALID << ShiftNATevent) & MasNATevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"add", 3) == 0 || strncasecmp(yystack.l_mark[0].s,"create", 6) == 0) {
			yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( NEL_EVENT_ADD << ShiftNATevent) & MasNATevent, CMP_EQ, FUNC_NONE, NULL );
		} else if( strncasecmp(yystack.l_mark[0].s,"delete", 6) == 0) {
			yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( NEL_EVENT_DELETE << ShiftNATevent) & MasNATevent, CMP_EQ, FUNC_NONE, NULL );
		} else {
			yyerror("Unknown nat event");
			YYABORT;
		}
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 2977 "grammar.c"
break;
case 62:
#line 1178 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Invalid event ID");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetNATevent, MasNATevent, ( yystack.l_mark[0].value << ShiftNATevent) & MasNATevent, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 2993 "grammar.c"
break;
case 63:
#line 1193 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockStart, ( yystack.l_mark[0].value << ShiftPortBlockStart) & MaskPortBlockStart, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 3009 "grammar.c"
break;
case 64:
#line 1206 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockEnd, ( yystack.l_mark[0].value << ShiftPortBlockEnd) & MaskPortBlockEnd, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 3025 "grammar.c"
break;
case 65:
#line 1219 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockStep, ( yystack.l_mark[0].value << ShiftPortBlockStep) & MaskPortBlockStep, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 3041 "grammar.c"
break;
case 66:
#line 1232 "grammar.y"
	{
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65536 ) {
			yyerror("Invalid port");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPortBlock, MaskPortBlockSize, ( yystack.l_mark[0].value << ShiftPortBlockSize) & MaskPortBlockSize, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
#else
		yyerror("NAT filters not available");
		YYABORT;
#endif
	}
#line 3057 "grammar.c"
break;
case 67:
#line 1245 "grammar.y"
	{	
#ifdef NSEL
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("Port outside of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATEPort, MaskXLATESRCPORT, (yystack.l_mark[0].value << ShiftXLATESRCPORT) & MaskXLATESRCPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetXLATEPort, MaskXLATEDSTPORT, (yystack.l_mark[0].value << ShiftXLATEDSTPORT) & MaskXLATEDSTPORT, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/
#else
		yyerror("NEL/NAT filters not available");
		YYABORT;
#endif

	}
#line 3098 "grammar.c"
break;
case 68:
#line 1283 "grammar.y"
	{ 	
#ifdef NSEL
		int af, bytes, ret;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Error parsing IP address.");
			YYABORT;
		}

		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		if ( ret == -2 ) {
			/* could not resolv host => 'not any'*/
			yyval.param.self = Invert(NewBlock(OffsetProto, 0, 0, CMP_EQ, FUNC_NONE, NULL )); 
		} else {
			uint64_t offsets[4] = {OffsetXLATESRCv6a, OffsetXLATESRCv6b, OffsetXLATEDSTv6a, OffsetXLATEDSTv6b };
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			switch ( yystack.l_mark[-2].param.direction ) {
				case SOURCE:
				case DESTINATION:
					yyval.param.self = ChainHosts(offsets, IPstack, num_ip, yystack.l_mark[-2].param.direction);
					break;
				case DIR_UNSPEC:
				case SOURCE_OR_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_OR(src, dst);
					} break;
				case SOURCE_AND_DESTINATION: {
					uint32_t src = ChainHosts(offsets, IPstack, num_ip, SOURCE);
					uint32_t dst = ChainHosts(offsets, IPstack, num_ip, DESTINATION);
					yyval.param.self = Connect_AND(src, dst);
					} break;
				default:
					yyerror("This token is not expected here!");
					YYABORT;
	
			} /* End of switch*/

		}
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 3152 "grammar.c"
break;
case 69:
#line 1335 "grammar.y"
	{
		if (strlen(yystack.l_mark[0].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_PAYLOAD, FUNC_NONE, yystack.l_mark[0].s); 
	}
#line 3163 "grammar.c"
break;
case 70:
#line 1343 "grammar.y"
	{
		if (strlen(yystack.l_mark[0].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		char *word = stripWord(yystack.l_mark[0].s);
		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_PAYLOAD, FUNC_NONE, word); 
	}
#line 3175 "grammar.c"
break;
case 71:
#line 1352 "grammar.y"
	{
		if (strlen(yystack.l_mark[0].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		/* strip ' or " */
		char *word = stripWord(yystack.l_mark[0].s);

		int err[2];
		srx_Context *program = srx_CreateExt(word, strlen(word), "", err, NULL, NULL);
		if ( !program ) {
			yyerror("failed to compile regex");
		}

		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_REGEX, FUNC_NONE, (char *)program); 
	}
#line 3195 "grammar.c"
break;
case 72:
#line 1369 "grammar.y"
	{
		if (strlen(yystack.l_mark[-1].s)>64) {
			yyerror("word too long");
			YYABORT;
		}
		/* strip ' or " */
		char *word = stripWord(yystack.l_mark[-1].s);

		int err[2];
		srx_Context *program = srx_CreateExt(word, strlen(word), yystack.l_mark[0].s, err, NULL, NULL);
		if ( !program ) {
			yyerror("failed to compile regex");
		}

		yyval.param.self = NewBlock(OffsetPayload, 0, 0, CMP_REGEX, FUNC_NONE, (char *)program); 
	}
#line 3215 "grammar.c"
break;
case 73:
#line 1386 "grammar.y"
	{
		union {
			uint8_t u8[16];
			uint64_t u64[2];
		} ja3;

		if ( strcasecmp(yystack.l_mark[0].s, "defined") == 0) {
			ja3.u64[0] = 0;
			ja3.u64[1] = 0;
			yyval.param.self = Invert(Connect_AND(
				NewBlock(OffsetJA3, MaskJA3, ja3.u64[0], CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(OffsetJA3+1, MaskJA3, ja3.u64[1], CMP_EQ, FUNC_NONE, NULL )
			));
		} else {
			if (!IsMD5(yystack.l_mark[0].s)) {
				yyerror("not a ja3 hash");
				YYABORT;
			}

			char *pos = yystack.l_mark[0].s;
			for(int count = 0; count < 16; count++) {
				sscanf(pos, "%2hhx", &ja3.u8[count]);
				pos += 2;
			}
			yyval.param.self = Connect_AND(
				NewBlock(OffsetJA3, MaskJA3, ja3.u64[0], CMP_EQ, FUNC_NONE, NULL ),
				NewBlock(OffsetJA3+1, MaskJA3, ja3.u64[1], CMP_EQ, FUNC_NONE, NULL )
			);
		}
		ja3Filter = 1;
	}
#line 3250 "grammar.c"
break;
case 74:
#line 1418 "grammar.y"
	{ 	
#ifdef NSEL
		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetXLATESRCv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetXLATEDSTv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetXLATESRCv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetXLATEDSTv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetXLATESRCv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetXLATEDSTv6a, MaskIPv6, 0 , CMP_IPLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		}
#else
		yyerror("NSEL/ASA filters not available");
		YYABORT;
#endif
	}
#line 3285 "grammar.c"
break;
case 75:
#line 1450 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 0xfFFFFFFF ) {
			yyerror("AS number of range");
			YYABORT;
		}

		geoFilter = 1;
		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetAS, MaskSrcAS, (yystack.l_mark[0].value << ShiftSrcAS) & MaskSrcAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetAS, MaskDstAS, (yystack.l_mark[0].value << ShiftDstAS) & MaskDstAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetAS, MaskSrcAS, (yystack.l_mark[0].value << ShiftSrcAS) & MaskSrcAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetAS, MaskDstAS, (yystack.l_mark[0].value << ShiftDstAS) & MaskDstAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetAS, MaskSrcAS, (yystack.l_mark[0].value << ShiftSrcAS) & MaskSrcAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL ),
					NewBlock(OffsetAS, MaskDstAS, (yystack.l_mark[0].value << ShiftDstAS) & MaskDstAS, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL)
				);
				break;
			case ADJ_PREV:
				yyval.param.self = NewBlock(OffsetBGPadj, MaskBGPadjPrev, (yystack.l_mark[0].value << ShiftBGPadjPrev) & MaskBGPadjPrev, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			case ADJ_NEXT:
				yyval.param.self = NewBlock(OffsetBGPadj, MaskBGPadjNext, (yystack.l_mark[0].value << ShiftBGPadjNext) & MaskBGPadjNext, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 3328 "grammar.c"
break;
case 76:
#line 1490 "grammar.y"
	{	
		if ( strlen(yystack.l_mark[0].s) != 2 ) {
			yyerror("Need a two letter geo country code");
			YYABORT;
		}

		geoFilter = 2;
		union {
			char c[8];
			uint64_t u;
		} v;
		v.u = 0;
#ifdef WORDS_BIGENDIAN
                v.c[4] = toupper(yystack.l_mark[0].s[0]);
                v.c[5] = toupper(yystack.l_mark[0].s[1]);
#else
                v.c[0] = toupper(yystack.l_mark[0].s[0]);
                v.c[1] = toupper(yystack.l_mark[0].s[1]);
#endif
		switch ( yystack.l_mark[-2].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetGeo, MaskSrcGeo, (v.u << ShiftSrcGeo) & MaskSrcGeo, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetGeo, MaskDstGeo, (v.u << ShiftDstGeo) & MaskDstGeo, CMP_EQ, FUNC_NONE, NULL);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetGeo, MaskSrcGeo, (v.u << ShiftSrcGeo) & MaskSrcGeo, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetGeo, MaskDstGeo, (v.u << ShiftDstGeo) & MaskDstGeo, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetGeo, MaskSrcGeo, (v.u << ShiftSrcGeo) & MaskSrcGeo, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetGeo, MaskDstGeo, (v.u << ShiftDstGeo) & MaskDstGeo, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 3377 "grammar.c"
break;
case 77:
#line 1536 "grammar.y"
	{ 	
		struct ULongListNode *node;
		ULongtree_t *root = NULL;

		geoFilter = 1;
		if ( yystack.l_mark[-5].param.direction == DIR_UNSPEC || yystack.l_mark[-5].param.direction == SOURCE_OR_DESTINATION || yystack.l_mark[-5].param.direction == SOURCE_AND_DESTINATION ) {
			/* src and/or dst AS*/
			/* we need a second rbtree due to different shifts for src and dst AS*/
			root = malloc(sizeof(ULongtree_t));

			struct ULongListNode *n;
			if ( root == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}
			RB_INIT(root);

			RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
				if ( node->value > 0xFFFFFFFFLL ) {
					yyerror("AS number of range");
					YYABORT;
				}
				if ((n = malloc(sizeof(struct ULongListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				n->value 	= (node->value << ShiftDstAS) & MaskDstAS;
				node->value = (node->value << ShiftSrcAS) & MaskSrcAS;
				RB_INSERT(ULongtree, root, n);
			}
		}

		switch ( yystack.l_mark[-5].param.direction ) {
			case SOURCE:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftSrcAS) & MaskSrcAS;
				}
				yyval.param.self = NewBlock(OffsetAS, MaskSrcAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DESTINATION:
				RB_FOREACH(node, ULongtree, (ULongtree_t *)yystack.l_mark[-1].list) {
					node->value = (node->value << ShiftDstAS) & MaskDstAS;
				}
				yyval.param.self = NewBlock(OffsetAS, MaskDstAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetAS, MaskSrcAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetAS, MaskDstAS, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetAS, MaskSrcAS, 0, CMP_ULLIST, FUNC_NONE, (void *)yystack.l_mark[-1].list ),
					NewBlock(OffsetAS, MaskDstAS, 0, CMP_ULLIST, FUNC_NONE, (void *)root )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		}

	}
#line 3445 "grammar.c"
break;
case 78:
#line 1601 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Mask outside of range 0..255");
			YYABORT;
		}

		switch ( yystack.l_mark[-2].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetMask, MaskSrcMask, (yystack.l_mark[0].value << ShiftSrcMask) & MaskSrcMask, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetMask, MaskDstMask, (yystack.l_mark[0].value << ShiftDstMask) & MaskDstMask, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetMask, MaskSrcMask, (yystack.l_mark[0].value << ShiftSrcMask) & MaskSrcMask, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetMask, MaskDstMask, (yystack.l_mark[0].value << ShiftDstMask) & MaskDstMask, CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetMask, MaskSrcMask, (yystack.l_mark[0].value << ShiftSrcMask) & MaskSrcMask, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetMask, MaskDstMask, (yystack.l_mark[0].value << ShiftDstMask) & MaskDstMask, CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End switch*/

	}
#line 3481 "grammar.c"
break;
case 79:
#line 1634 "grammar.y"
	{ 
		int af, bytes, ret;
		uint64_t	mask[2];
		ret = parse_ip(&af, yystack.l_mark[-1].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af != PF_INET ) {
			yyerror("IP netmask syntax valid only for IPv4");
			YYABORT;
		}
		if ( bytes != 4 ) {
			yyerror("Need complete IP address");
			YYABORT;
		}

		ret = parse_ip(&af, yystack.l_mark[0].s, mask, &bytes, STRICT_IP, &num_ip);
		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/

		if ( af != PF_INET || bytes != 4 ) {
			yyerror("Invalid netmask for IPv4 address");
			YYABORT;
		}

		IPstack[0] &= mask[0];
		IPstack[1] &= mask[1];

		switch ( yystack.l_mark[-3].param.direction ) {
			case SOURCE:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);		
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			default:
				/* should never happen */
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 3574 "grammar.c"
break;
case 80:
#line 1724 "grammar.y"
	{ 
		int af, bytes, ret;
		uint64_t	mask[2];

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);
		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( ret == -1 ) {
			yyerror("IP address required - hostname not allowed here.");
			YYABORT;
		}
		/* ret == -2 will never happen here, as STRICT_IP is set*/


		if ( yystack.l_mark[0].value > (bytes*8) ) {
			yyerror("Too many netbits for this IP address");
			YYABORT;
		}

		if ( af == PF_INET ) {
			mask[0] = 0xffffffffffffffffLL;
			mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
		} else {	/* PF_INET6*/
			if ( yystack.l_mark[0].value > 64 ) {
				mask[0] = 0xffffffffffffffffLL;
				mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
			} else {
				mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
				mask[1] = 0;
			}
		}
		/* IP aadresses are stored in network representation */
		mask[0]	 = mask[0];
		mask[1]	 = mask[1];

		IPstack[0] &= mask[0];
		IPstack[1] &= mask[1];

		switch ( yystack.l_mark[-4].param.direction ) {
			case SOURCE:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
				);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					Connect_AND(
						NewBlock(OffsetSrcIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetSrcIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					),
					Connect_AND(
						NewBlock(OffsetDstIPv6b, mask[1], IPstack[1] , CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetDstIPv6a, mask[0], IPstack[0] , CMP_EQ, FUNC_NONE, NULL )
					)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 3662 "grammar.c"
break;
case 81:
#line 1809 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 0xffffffffLL ) {
			yyerror("Input interface number must 0..2^32");
			YYABORT;
		}

		switch ( yystack.l_mark[-2].param.direction ) {
			case DIR_UNSPEC:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetInOut, MaskInput, (yystack.l_mark[0].value << ShiftInput) & MaskInput, CMP_EQ, FUNC_NONE, NULL),
					NewBlock(OffsetInOut, MaskOutput, (yystack.l_mark[0].value << ShiftOutput) & MaskOutput, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			case DIR_IN: 
				yyval.param.self = NewBlock(OffsetInOut, MaskInput, (yystack.l_mark[0].value << ShiftInput) & MaskInput, CMP_EQ, FUNC_NONE, NULL); 
				break;
			case DIR_OUT: 
				yyval.param.self = NewBlock(OffsetInOut, MaskOutput, (yystack.l_mark[0].value << ShiftOutput) & MaskOutput, CMP_EQ, FUNC_NONE, NULL); 
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 3691 "grammar.c"
break;
case 82:
#line 1835 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > 65535 ) {
			yyerror("VLAN number of range 0..65535");
			YYABORT;
		}

		switch ( yystack.l_mark[-2].param.direction ) {
			case SOURCE:
				yyval.param.self = NewBlock(OffsetVlan, MaskSrcVlan, (yystack.l_mark[0].value << ShiftSrcVlan) & MaskSrcVlan, CMP_EQ, FUNC_NONE, NULL );
				break;
			case DESTINATION:
				yyval.param.self = NewBlock(OffsetVlan, MaskDstVlan, (yystack.l_mark[0].value << ShiftDstVlan) & MaskDstVlan, CMP_EQ, FUNC_NONE, NULL);
				break;
			case DIR_UNSPEC:
			case SOURCE_OR_DESTINATION:
				yyval.param.self = Connect_OR(
					NewBlock(OffsetVlan, MaskSrcVlan, (yystack.l_mark[0].value << ShiftSrcVlan) & MaskSrcVlan, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetVlan, MaskDstVlan, (yystack.l_mark[0].value << ShiftDstVlan) & MaskDstVlan, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			case SOURCE_AND_DESTINATION:
				yyval.param.self = Connect_AND(
					NewBlock(OffsetVlan, MaskSrcVlan, (yystack.l_mark[0].value << ShiftSrcVlan) & MaskSrcVlan, CMP_EQ, FUNC_NONE, NULL ),
					NewBlock(OffsetVlan, MaskDstVlan, (yystack.l_mark[0].value << ShiftDstVlan) & MaskDstVlan, CMP_EQ, FUNC_NONE, NULL)
				);
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/

	}
#line 3727 "grammar.c"
break;
case 83:
#line 1868 "grammar.y"
	{
		uint64_t	mac = VerifyMac(yystack.l_mark[0].s);
		if ( mac == 0 ) {
			yyerror("Invalid MAC address format");
			YYABORT;
		}
		switch ( yystack.l_mark[-2].param.direction ) {
			case DIR_UNSPEC: {
					uint32_t in, out;
					in  = Connect_OR(
						NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					out  = Connect_OR(
						NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					yyval.param.self = Connect_OR(in, out);
					} break;
			case DIR_IN:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case DIR_OUT:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case SOURCE:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case DESTINATION:
					yyval.param.self = Connect_OR(
						NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL ),
						NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL )
					);
					break;
			case IN_SRC: 
					yyval.param.self = NewBlock(OffsetInSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
			case IN_DST: 
					yyval.param.self = NewBlock(OffsetInDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
			case OUT_SRC: 
					yyval.param.self = NewBlock(OffsetOutSrcMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
			case OUT_DST:
					yyval.param.self = NewBlock(OffsetOutDstMAC, MaskMac, mac, CMP_EQ, FUNC_NONE, NULL );
					break;
				break;
			default:
				yyerror("This token is not expected here!");
				YYABORT;
		} /* End of switch*/
	}
#line 3792 "grammar.c"
break;
case 84:
#line 1930 "grammar.y"
	{	
		if ( yystack.l_mark[0].value > MPLSMAX ) {
			yyerror("MPLS value out of range");
			YYABORT;
		}

		/* search for label1 - label10*/
		if ( strncasecmp(yystack.l_mark[-2].s, "label", 5) == 0 ) {
			uint64_t mask;
			uint32_t offset, shift;
			char *s = &yystack.l_mark[-2].s[5];
			if ( *s == '\0' ) {
				yyerror("Missing label number");
				YYABORT;
			}
			int i = (int)strtol(s, (char **)NULL, 10);

			switch (i) {
				case 1:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 2:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 3:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 4:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 5:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 6:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 7:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 8:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				case 9:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSlabelOdd;
					shift	= ShiftMPLSlabelOdd;
					break;
				case 10:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSlabelEven;
					shift	= ShiftMPLSlabelEven;
					break;
				default: 
					yyerror("MPLS label out of range 1..10");
					YYABORT;
			}
			yyval.param.self = NewBlock(offset, mask, (yystack.l_mark[0].value << shift) & mask, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );

		} else if ( strcasecmp(yystack.l_mark[-2].s, "eos") == 0 ) {
			/* match End of Stack label */
			yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value << 4, yystack.l_mark[-1].param.comp, FUNC_MPLS_EOS, NULL );

		} else if ( strncasecmp(yystack.l_mark[-2].s, "exp", 3) == 0 ) {
			uint64_t mask;
			uint32_t offset, shift;
			char *s = &yystack.l_mark[-2].s[3];
			if ( *s == '\0' ) {
				yyerror("Missing label number");
				YYABORT;
			}
			int i = (int)strtol(s, (char **)NULL, 10);

			if ( yystack.l_mark[0].value > 7 ) {
				yyerror("MPLS exp value out of range");
				YYABORT;
			}

			switch (i) {
				case 1:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 2:
					offset	= OffsetMPLS12;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 3:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 4:
					offset	= OffsetMPLS34;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 5:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 6:
					offset	= OffsetMPLS56;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 7:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 8:
					offset	= OffsetMPLS78;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				case 9:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSexpOdd;
					shift	= ShiftMPLSexpOdd;
					break;
				case 10:
					offset	= OffsetMPLS910;
					mask	= MaskMPLSexpEven;
					shift	= ShiftMPLSexpEven;
					break;
				default: 
					yyerror("MPLS label out of range 1..10");
					YYABORT;
			}
			yyval.param.self = NewBlock(offset, mask, yystack.l_mark[0].value << shift, yystack.l_mark[-1].param.comp, FUNC_NONE, NULL );

		} else {
			yyerror("Unknown MPLS option");
			YYABORT;
		}
	}
#line 3951 "grammar.c"
break;
case 85:
#line 2085 "grammar.y"
	{	
		uint32_t *opt = malloc(sizeof(uint32_t));
		if ( yystack.l_mark[0].value > MPLSMAX ) {
			yyerror("MPLS value out of range");
			YYABORT;
		}
		if ( opt == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		*opt = yystack.l_mark[0].value << 4;
		yyval.param.self = NewBlock(0, AnyMask, yystack.l_mark[0].value << 4, CMP_EQ, FUNC_MPLS_ANY, opt );

	}
#line 3969 "grammar.c"
break;
case 86:
#line 2099 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 255 ) {
			yyerror("Forwarding status of range 0..255");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetStatus, MaskStatus, (yystack.l_mark[0].value << ShiftStatus) & MaskStatus, CMP_EQ, FUNC_NONE, NULL);
	}
#line 3980 "grammar.c"
break;
case 87:
#line 2107 "grammar.y"
	{
		uint64_t id = Get_fwd_status_id(yystack.l_mark[0].s);
		if (id == 256 ) {
			yyerror("Unknown forwarding status");
			YYABORT;
		}

		yyval.param.self = NewBlock(OffsetStatus, MaskStatus, (id << ShiftStatus) & MaskStatus, CMP_EQ, FUNC_NONE, NULL);

	}
#line 3994 "grammar.c"
break;
case 88:
#line 2118 "grammar.y"
	{
		if ( yystack.l_mark[0].value > 2 ) {
			yyerror("Flow direction status of range 0, 1");
			YYABORT;
		}
		yyval.param.self = NewBlock(OffsetDir, MaskDir, (yystack.l_mark[0].value << ShiftDir) & MaskDir, CMP_EQ, FUNC_NONE, NULL);

	}
#line 4006 "grammar.c"
break;
case 89:
#line 2127 "grammar.y"
	{
		uint64_t dir = 0xFF;
		if ( yystack.l_mark[0].param.inout == INGRESS )
			dir = 0;
		else if ( yystack.l_mark[0].param.inout == EGRESS )
			dir = 1;
		else {
			yyerror("Flow direction status of range ingress, egress");
			YYABORT;
		}

		yyval.param.self = NewBlock(OffsetDir, MaskDir, (dir << ShiftDir) & MaskDir, CMP_EQ, FUNC_NONE, NULL);

	}
#line 4024 "grammar.c"
break;
case 90:
#line 2142 "grammar.y"
	{
		yyval.param.self = NewBlock(OffsetObservationDomainID, MaskObservationDomainID, (yystack.l_mark[0].value << ShiftObservationDomainID) & MaskObservationDomainID, CMP_EQ, FUNC_NONE, NULL);
	}
#line 4031 "grammar.c"
break;
case 91:
#line 2146 "grammar.y"
	{
		yyval.param.self = NewBlock(OffsetObservationPointID, MaskObservationPointID, (yystack.l_mark[0].value << ShiftObservationPointID) & MaskObservationPointID, CMP_EQ, FUNC_NONE, NULL);
	}
#line 4038 "grammar.c"
break;
case 92:
#line 2151 "grammar.y"
	{ 
		int i, af, bytes, ret;
		struct IPListNode *node;

		IPlist_t *root = malloc(sizeof(IPlist_t));

		if ( root == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		RB_INIT(root);

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		
		if ( ret != -2 ) {
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			for ( i=0; i<num_ip; i++ ) {
				if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				node->ip[0] = IPstack[2*i];
				node->ip[1] = IPstack[2*i+1];
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL;
				RB_INSERT(IPtree, root, node);
			}

		}
		yyval.list = (void *)root;

	}
#line 4084 "grammar.c"
break;
case 93:
#line 2194 "grammar.y"
	{ 
		int af, bytes, ret;
		struct IPListNode *node;

		IPlist_t *root = malloc(sizeof(IPlist_t));

		if ( root == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		RB_INIT(root);

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		/* ret == -1 will never happen here, as ALLOW_LOOKUP is set*/
		
		if ( ret != -2 ) {
			if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
				yyerror("incomplete IP address");
				YYABORT;
			}

			if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}

			if ( af == PF_INET ) {
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
			} else {	/* PF_INET6*/
				if ( yystack.l_mark[0].value > 64 ) {
					node->mask[0] = 0xffffffffffffffffLL;
					node->mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
				} else {
					node->mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
					node->mask[1] = 0;
				}
			}

			node->ip[0] = IPstack[0] & node->mask[0];
			node->ip[1] = IPstack[1] & node->mask[1];

			RB_INSERT(IPtree, root, node);

		}
		yyval.list = (void *)root;

	}
#line 4141 "grammar.c"
break;
case 94:
#line 2248 "grammar.y"
	{ 
		int i, af, bytes, ret;
		struct IPListNode *node;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		/* ret == - 2 means lookup failure*/
		if ( ret != -2 ) {
			for ( i=0; i<num_ip; i++ ) {
				if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				node->ip[0] = IPstack[2*i];
				node->ip[1] = IPstack[2*i+1];
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL;
	
				RB_INSERT(IPtree, (IPlist_t *)yyval.list, node);
			}
		}
	}
#line 4176 "grammar.c"
break;
case 95:
#line 2279 "grammar.y"
	{ 
		int i, af, bytes, ret;
		struct IPListNode *node;

		ret = parse_ip(&af, yystack.l_mark[0].s, IPstack, &bytes, ALLOW_LOOKUP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		/* ret == - 2 means lookup failure*/
		if ( ret != -2 ) {
			for ( i=0; i<num_ip; i++ ) {
				if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
					yyerror("malloc() error");
					YYABORT;
				}
				node->ip[0] = IPstack[2*i];
				node->ip[1] = IPstack[2*i+1];
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL;
	
				RB_INSERT(IPtree, (IPlist_t *)yyval.list, node);
			}
		}
	}
#line 4211 "grammar.c"
break;
case 96:
#line 2311 "grammar.y"
	{ 
		int af, bytes, ret;
		struct IPListNode *node;

		ret = parse_ip(&af, yystack.l_mark[-2].s, IPstack, &bytes, STRICT_IP, &num_ip);

		if ( ret == 0 ) {
			yyerror("Invalid IP address");
			YYABORT;
		}
		if ( af && (( af == PF_INET && bytes != 4 ) || ( af == PF_INET6 && bytes != 16 ))) {
			yyerror("incomplete IP address");
			YYABORT;
		}

		/* ret == - 2 means lookup failure*/
		if ( ret != -2 ) {
			if ((node = malloc(sizeof(struct IPListNode))) == NULL) {
				yyerror("malloc() error");
				YYABORT;
			}
			if ( af == PF_INET ) {
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL << ( 32 - yystack.l_mark[0].value );
			} else {	/* PF_INET6*/
				if ( yystack.l_mark[0].value > 64 ) {
					node->mask[0] = 0xffffffffffffffffLL;
					node->mask[1] = 0xffffffffffffffffLL << ( 128 - yystack.l_mark[0].value );
				} else {
					node->mask[0] = 0xffffffffffffffffLL << ( 64 - yystack.l_mark[0].value );
					node->mask[1] = 0;
				}
			}

			node->ip[0] = IPstack[0] & node->mask[0];
			node->ip[1] = IPstack[1] & node->mask[1];

			RB_INSERT(IPtree, (IPlist_t *)yyval.list, node);
		}
	}
#line 4255 "grammar.c"
break;
case 97:
#line 2355 "grammar.y"
	{ 
		struct ULongListNode *node;

		ULongtree_t *root = malloc(sizeof(ULongtree_t));

		if ( root == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		RB_INIT(root);

		if ((node = malloc(sizeof(struct ULongListNode))) == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		node->value = yystack.l_mark[0].value;

		RB_INSERT(ULongtree, root, node);
		yyval.list = (void *)root;
	}
#line 4279 "grammar.c"
break;
case 98:
#line 2376 "grammar.y"
	{ 
		struct ULongListNode *node;

		if ((node = malloc(sizeof(struct ULongListNode))) == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		node->value = yystack.l_mark[0].value;
		RB_INSERT(ULongtree, (ULongtree_t *)yyval.list, node);
	}
#line 4293 "grammar.c"
break;
case 99:
#line 2387 "grammar.y"
	{ 
		struct ULongListNode *node;

		if ((node = malloc(sizeof(struct ULongListNode))) == NULL) {
			yyerror("malloc() error");
			YYABORT;
		}
		node->value = yystack.l_mark[0].value;
		RB_INSERT(ULongtree, (ULongtree_t *)yyval.list, node);
	}
#line 4307 "grammar.c"
break;
case 100:
#line 2401 "grammar.y"
	{ yyval.param.comp = CMP_EQ; }
#line 4312 "grammar.c"
break;
case 101:
#line 2402 "grammar.y"
	{ yyval.param.comp = CMP_EQ; }
#line 4317 "grammar.c"
break;
case 102:
#line 2403 "grammar.y"
	{ yyval.param.comp = CMP_LT; }
#line 4322 "grammar.c"
break;
case 103:
#line 2404 "grammar.y"
	{ yyval.param.comp = CMP_GT; }
#line 4327 "grammar.c"
break;
case 104:
#line 2405 "grammar.y"
	{ yyval.param.comp = CMP_LE; }
#line 4332 "grammar.c"
break;
case 105:
#line 2406 "grammar.y"
	{ yyval.param.comp = CMP_GE; }
#line 4337 "grammar.c"
break;
case 106:
#line 2410 "grammar.y"
	{ yyval.param.direction = DIR_UNSPEC;  			 }
#line 4342 "grammar.c"
break;
case 107:
#line 2411 "grammar.y"
	{ yyval.param.direction = SOURCE;				 }
#line 4347 "grammar.c"
break;
case 108:
#line 2412 "grammar.y"
	{ yyval.param.direction = DESTINATION;			 }
#line 4352 "grammar.c"
break;
case 109:
#line 2413 "grammar.y"
	{ yyval.param.direction = SOURCE_OR_DESTINATION;  }
#line 4357 "grammar.c"
break;
case 110:
#line 2414 "grammar.y"
	{ yyval.param.direction = SOURCE_OR_DESTINATION;  }
#line 4362 "grammar.c"
break;
case 111:
#line 2415 "grammar.y"
	{ yyval.param.direction = SOURCE_AND_DESTINATION; }
#line 4367 "grammar.c"
break;
case 112:
#line 2416 "grammar.y"
	{ yyval.param.direction = SOURCE_AND_DESTINATION; }
#line 4372 "grammar.c"
break;
case 113:
#line 2417 "grammar.y"
	{ yyval.param.direction = DIR_IN;				 }
#line 4377 "grammar.c"
break;
case 114:
#line 2418 "grammar.y"
	{ yyval.param.direction = DIR_OUT;				 }
#line 4382 "grammar.c"
break;
case 115:
#line 2419 "grammar.y"
	{ yyval.param.direction = IN_SRC;				 }
#line 4387 "grammar.c"
break;
case 116:
#line 2420 "grammar.y"
	{ yyval.param.direction = IN_DST;				 }
#line 4392 "grammar.c"
break;
case 117:
#line 2421 "grammar.y"
	{ yyval.param.direction = OUT_SRC;				 }
#line 4397 "grammar.c"
break;
case 118:
#line 2422 "grammar.y"
	{ yyval.param.direction = OUT_DST;				 }
#line 4402 "grammar.c"
break;
case 119:
#line 2423 "grammar.y"
	{ yyval.param.direction = ADJ_PREV;				 }
#line 4407 "grammar.c"
break;
case 120:
#line 2424 "grammar.y"
	{ yyval.param.direction = ADJ_NEXT;				 }
#line 4412 "grammar.c"
break;
case 121:
#line 2427 "grammar.y"
	{ yyval.param.inout	= INGRESS;	}
#line 4417 "grammar.c"
break;
case 122:
#line 2428 "grammar.y"
	{ yyval.param.inout	= EGRESS;   }
#line 4422 "grammar.c"
break;
case 123:
#line 2431 "grammar.y"
	{ yyval.param.acl = ACL; 	}
#line 4427 "grammar.c"
break;
case 124:
#line 2432 "grammar.y"
	{ yyval.param.acl = ACE;		}
#line 4432 "grammar.c"
break;
case 125:
#line 2433 "grammar.y"
	{ yyval.param.acl = XACE;	}
#line 4437 "grammar.c"
break;
case 126:
#line 2436 "grammar.y"
	{ yyval.value = yystack.l_mark[0].param.self;        }
#line 4442 "grammar.c"
break;
case 127:
#line 2437 "grammar.y"
	{ yyval.value = Connect_OR(yystack.l_mark[-2].value, yystack.l_mark[0].value);  }
#line 4447 "grammar.c"
break;
case 128:
#line 2438 "grammar.y"
	{ yyval.value = Connect_AND(yystack.l_mark[-2].value, yystack.l_mark[0].value); }
#line 4452 "grammar.c"
break;
case 129:
#line 2439 "grammar.y"
	{ yyval.value = Invert(yystack.l_mark[0].value);			}
#line 4457 "grammar.c"
break;
case 130:
#line 2440 "grammar.y"
	{ yyval.value = yystack.l_mark[-1].value; }
#line 4462 "grammar.c"
break;
case 131:
#line 2441 "grammar.y"
	{ 
		yyval.value = yystack.l_mark[-3].value; 
		if ( strlen(yystack.l_mark[0].s) > 16 ) {
			yyerror("Error: Maximum 16 chars allowed for flowlabel");
			YYABORT;
		} else {
			AddLabel(yystack.l_mark[-3].value, yystack.l_mark[0].s);
		}
	}
#line 4475 "grammar.c"
break;
case 132:
#line 2450 "grammar.y"
	{ 
		yyval.value = yystack.l_mark[-1].value; 
		if ( strlen(yystack.l_mark[-3].s) > 16 ) {
			yyerror("Error: Maximum 16 chars allowed for flowlabel");
			YYABORT;
		} else {
			AddLabel(yystack.l_mark[-1].value, yystack.l_mark[-3].s);
		}
	}
#line 4488 "grammar.c"
break;
#line 4490 "grammar.c"
    default:
        break;
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark -= yym;
#endif
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
        {
            fprintf(stderr, "%s[%d]: after reduction, ", YYDEBUGSTR, yydepth);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
            if (!yytrial)
#endif /* YYBTYACC */
                fprintf(stderr, "result is <%s>, ", YYSTYPE_TOSTRING(yystos[YYFINAL], yyval));
#endif
            fprintf(stderr, "shifting from state 0 to final state %d\n", YYFINAL);
        }
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        *++yystack.p_mark = yyloc;
#endif
        if (yychar < 0)
        {
#if YYBTYACC
            do {
            if (yylvp < yylve)
            {
                /* we're currently re-reading tokens */
                yylval = *yylvp++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylloc = *yylpp++;
#endif
                yychar = *yylexp++;
                break;
            }
            if (yyps->save)
            {
                /* in trial mode; save scanner results for future parse attempts */
                if (yylvp == yylvlim)
                {   /* Enlarge lexical value queue */
                    size_t p = (size_t) (yylvp - yylvals);
                    size_t s = (size_t) (yylvlim - yylvals);

                    s += YYLVQUEUEGROWTH;
                    if ((yylexemes = (YYINT *)realloc(yylexemes, s * sizeof(YYINT))) == NULL)
                        goto yyenomem;
                    if ((yylvals   = (YYSTYPE *)realloc(yylvals, s * sizeof(YYSTYPE))) == NULL)
                        goto yyenomem;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    if ((yylpsns   = (YYLTYPE *)realloc(yylpsns, s * sizeof(YYLTYPE))) == NULL)
                        goto yyenomem;
#endif
                    yylvp   = yylve = yylvals + p;
                    yylvlim = yylvals + s;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    yylpp   = yylpe = yylpsns + p;
                    yylplim = yylpsns + s;
#endif
                    yylexp  = yylexemes + p;
                }
                *yylexp = (YYINT) YYLEX;
                *yylvp++ = yylval;
                yylve++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                *yylpp++ = yylloc;
                yylpe++;
#endif
                yychar = *yylexp++;
                break;
            }
            /* normal operation, no conflict encountered */
#endif /* YYBTYACC */
            yychar = YYLEX;
#if YYBTYACC
            } while (0);
#endif /* YYBTYACC */
            if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
            if (yydebug)
            {
                if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
                fprintf(stderr, "%s[%d]: state %d, reading token %d (%s)\n",
                                YYDEBUGSTR, yydepth, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == YYEOF) goto yyaccept;
        goto yyloop;
    }
    if (((yyn = yygindex[yym]) != 0) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
    {
        fprintf(stderr, "%s[%d]: after reduction, ", YYDEBUGSTR, yydepth);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
        if (!yytrial)
#endif /* YYBTYACC */
            fprintf(stderr, "result is <%s>, ", YYSTYPE_TOSTRING(yystos[yystate], yyval));
#endif
        fprintf(stderr, "shifting from state %d to state %d\n", *yystack.s_mark, yystate);
    }
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    *++yystack.s_mark = (YYINT) yystate;
    *++yystack.l_mark = yyval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    *++yystack.p_mark = yyloc;
#endif
    goto yyloop;
#if YYBTYACC

    /* Reduction declares that this path is valid. Set yypath and do a full parse */
yyvalid:
    if (yypath) YYABORT;
    while (yyps->save)
    {
        YYParseState *save = yyps->save;
        yyps->save = save->save;
        save->save = yypath;
        yypath = save;
    }
#if YYDEBUG
    if (yydebug)
        fprintf(stderr, "%s[%d]: state %d, CONFLICT trial successful, backtracking to state %d, %d tokens\n",
                        YYDEBUGSTR, yydepth, yystate, yypath->state, (int)(yylvp - yylvals - yypath->lexeme));
#endif
    if (yyerrctx)
    {
        yyFreeState(yyerrctx);
        yyerrctx = NULL;
    }
    yylvp          = yylvals + yypath->lexeme;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yylpp          = yylpsns + yypath->lexeme;
#endif
    yylexp         = yylexemes + yypath->lexeme;
    yychar         = YYEMPTY;
    yystack.s_mark = yystack.s_base + (yypath->yystack.s_mark - yypath->yystack.s_base);
    memcpy (yystack.s_base, yypath->yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
    yystack.l_mark = yystack.l_base + (yypath->yystack.l_mark - yypath->yystack.l_base);
    memcpy (yystack.l_base, yypath->yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark = yystack.p_base + (yypath->yystack.p_mark - yypath->yystack.p_base);
    memcpy (yystack.p_base, yypath->yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
    yystate        = yypath->state;
    goto yyloop;
#endif /* YYBTYACC */

yyoverflow:
    YYERROR_CALL("yacc stack overflow");
#if YYBTYACC
    goto yyabort_nomem;
yyenomem:
    YYERROR_CALL("memory exhausted");
yyabort_nomem:
#endif /* YYBTYACC */
    yyresult = 2;
    goto yyreturn;

yyabort:
    yyresult = 1;
    goto yyreturn;

yyaccept:
#if YYBTYACC
    if (yyps->save) goto yyvalid;
#endif /* YYBTYACC */
    yyresult = 0;

yyreturn:
#if defined(YYDESTRUCT_CALL)
    if (yychar != YYEOF && yychar != YYEMPTY)
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        YYDESTRUCT_CALL("cleanup: discarding token", yychar, &yylval, &yylloc);
#else
        YYDESTRUCT_CALL("cleanup: discarding token", yychar, &yylval);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */

    {
        YYSTYPE *pv;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        YYLTYPE *pp;

        for (pv = yystack.l_base, pp = yystack.p_base; pv <= yystack.l_mark; ++pv, ++pp)
             YYDESTRUCT_CALL("cleanup: discarding state",
                             yystos[*(yystack.s_base + (pv - yystack.l_base))], pv, pp);
#else
        for (pv = yystack.l_base; pv <= yystack.l_mark; ++pv)
             YYDESTRUCT_CALL("cleanup: discarding state",
                             yystos[*(yystack.s_base + (pv - yystack.l_base))], pv);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
    }
#endif /* defined(YYDESTRUCT_CALL) */

#if YYBTYACC
    if (yyerrctx)
    {
        yyFreeState(yyerrctx);
        yyerrctx = NULL;
    }
    while (yyps)
    {
        YYParseState *save = yyps;
        yyps = save->save;
        save->save = NULL;
        yyFreeState(save);
    }
    while (yypath)
    {
        YYParseState *save = yypath;
        yypath = save->save;
        save->save = NULL;
        yyFreeState(save);
    }
#endif /* YYBTYACC */
    yyfreestack(&yystack);
    return (yyresult);
}
